#!/usr/bin/perl

use strict;
use Cwd;
use Encode;

#########################################################################
# GemX internal variables     						#
# !!!DO NOT EDIT!!!           						#
$gx::dir="";              # Directory with GemX and its config files	#
$gx::xver=Xchat::get_info("version"); #					#
$gx::workdir="";                                                        #
$gx::xchat_required="2.8.0";
#########################################################################
# if (($^O =~ "bsd") or ($^O =~ "linux") or ($^O =~ "solaris")) { $OldDir = `pwd` }       # NEW
# if (($^O =~ "dos") or ($^O =~ "MSWin32")) { $OldDir = `cd` }                            # NEW
# $DotDot = '..';                                                                         # NEW
# if ($^O =~ "MacOS") { $DotDot = "::" }                                                  # NEW

my $version=Xchat::get_info("version");
#my $version="2.6.8a-2s";
#my $version="2.6.8";
#my $version="2.6.9a";
#my $version="2.6.8bob";

$version=~ /^(\d+\.\d+\.(\d+))(?:([a-z]?)(?:\s+|\-+|_+|\\+|\/+|$)(\w*)|(\w*$))/;
$version=$1;
$gx::xchat_official=$2%2;
my $rev=$3;
my $build;
$build=$5 if ($5);
$build=$4 if ($4);

$build="Official" if ($gx::xchat_official);

#Xchat::print("XChat version $version , Revision $rev, Build $build Official: $gx::xchat_official");

my $int_version = unpack "N", pack "C4", split /\./, $version;
my $int_required= unpack "N", pack "C4", split /\./, $gx::xchat_required;

if ($int_version < $int_required) {
	Xchat::emit_print("Generic Message","\00314.:.\003","GemX requires XChat version $gx::xchat_required or higher!");
	die();
}

#Xchat::emit_print("Generic Message","\00314.:.\003","GemX loading in XChat $version$rev ($build) - $gx::xchat_official");

Xchat::register("GemX: Main loader","$gx::ver","","");
Xchat::hook_timer(10000, "clean_start",,);

my $cfgdir = Xchat::get_info("xchatdir");
my $firstrun=("$cfgdir\/GemX\/gx-first.pl");
my $update=("$cfgdir\/GemX\/gx-update.pl");

$gx::workdir=Cwd::getcwd();
#Xchat::emit_print("Generic Message","\00314.:.\003","GemX workdir: ".$gx::workdir);
if ($^O =~ "MSWin32") {
					# Bugfix for Windows XP:
					# environment variable APPDATA is
					# wrong when using fast user switching
					# So get the X-Chat 2 config dir first...
	my $appdata;
	$appdata = Xchat::get_info("xchatdir");
	$gx::dir = "${appdata}\\GemX\\";
#	Xchat::emit_print("Generic Message","\00314.:.\003","GemX dir: ".$gx::dir);
	$appdata=~ s/\\X-Chat 2//i;      # strip "\X-Chat 2" to get true APPDATA
	$ENV{APPDATA} = $appdata;        # and write it to environment...

	if (-e "$firstrun") {
		Xchat::command("PL_RELOAD \"$firstrun\"");
		$firstrun=~ s/\//\\/g;
#		system("del \"$firstrun\"");
	}
	if (-e "$update") {
		Xchat::command("PL_RELOAD \"$update\"");
		$update=~ s/\//\\/g;
		system("del \"$update\"");
	}

	&load_main;			# Main script
	&load_win32;		# Platform abstraction layer for Windows 32
	&load_common;		# Common sub-scripts

} else {
	my $machtype = `uname`;

	$gx::dir = "$ENV{HOME}/.xchat2/GemX/";

	if (-e "$firstrun") {
		do $firstrun;
		system("rm \"$firstrun\"");
	}
	if (-e "$update") {
		do $update;
		system("rm \"$update\"");
	}

	&load_main;				# Main script

	&load_ix if ($^O =~ "linux");		# Platform acstraction layer for *ix
	&load_osx if ($^O =~ "MacOS");		# Platform acstraction layer for MacOS/x
	&load_common;				# Common sub-scripts
}

sub load_main {
	Xchat::command("PL_RELOAD \"${gx::dir}module.pl\"");
	Xchat::command("PL_RELOAD \"${gx::dir}main.pl\"");
	return Xchat::EAT_ALL;
}

sub load_win32 {
	Xchat::command("PL_RELOAD \"${gx::dir}win32.pl\"");
	return Xchat::EAT_ALL;
}

sub load_ix {
	Xchat::command("PL_RELOAD \"${gx::dir}unix.pl\"");
	return Xchat::EAT_ALL;
}

sub load_osx {
	Xchat::command("PL_RELOAD \"${gx::dir}osx.pl\"");
	return Xchat::EAT_ALL;
}

sub load_common {
	Xchat::command("PL_RELOAD \"${gx::dir}actplug.pl\"");
	Xchat::command("PL_RELOAD \"${gx::dir}mirc2.pl\"") if (-e "${gx::dir}mirc2.pl");
	Xchat::command("PL_RELOAD \"${gx::dir}dcc-ip.pl\"") if (-e "${gx::dir}dcc-ip.pl");
	Xchat::command("PL_RELOAD \"${gx::dir}marq.pl\"") if (-e "${gx::dir}marq.pl");
	return Xchat::EAT_ALL;
}

sub clean_start {
	my @dir_contents;
	my $file;
	my @dirlist=();

	$cfgdir = encode("iso-8859-15", $cfgdir);

	opendir(DIR,$cfgdir); # || die("Cannot open directory !\n");
		@dir_contents= readdir(DIR);
	closedir(DIR);

	my $found=0;
	foreach $file (@dir_contents) {
		if(!(($file eq ".") || ($file eq ".."))) {
			my @parts=split(/\./,"$file");
			my $ext=lc(@parts[$#parts]);
			if ((("$ext" eq "pl") || ("$ext" eq "py") || ("$ext" eq "tcl")) && (lc($file) ne "gemx.pl")) {
				push(@dirlist,$file);
				$found++;
				Xchat::print(".:. Found $file");
			}
		}
	}
	if ($found) {
		use File::Copy;
		if (!-d "$cfgdir\/backup.gx") {
			mkdir("$cfgdir\/backup.gx", 0777);
		}
		for my $n (0 .. ($#dirlist)) {
			my $oldspec=("$cfgdir\/@dirlist[$n]");
			my $newspec=("$cfgdir\/backup.gx\/@dirlist[$n]");
#			Xchat::command("UNLOAD \"@dirlist[$n]\"");
			move($oldspec, $newspec);
		}
		Xchat::command("TIMER 1 RELOADALL");
	} 
	return Xchat::REMOVE;
}
