#!/usr/bin/perl
#use Xchat qw(:all);

Xchat::register("GemX: Main Module","","",\&GemX::WriteConfig);

use strict;
use XML::Simple;
use LWP::UserAgent;
use Data::Dumper;
use Encode;
use utf8;
use Mail::POP3Client;

my $last_uidl;
my $proxy="81.189.106.138";

my $xs;
$xs = new XML::Simple(
	XMLDecl => 1,
	NumericEscape => 2,
	SuppressEmpty => 1,
	RootName => 'GemX',
	ForceArray => ['NETWORK', 'CHANNEL', 'USER'],
	KeyAttr => { USER => "NICK", CHANNEL => "NAME", NETWORK => "NAME" }
);

my $configdir=Xchat::get_info("xchatdir")."/GemX";
$configdir=~ s/\\{2}/\//g;
$configdir=~ s/\\/\//g;
my $charsets=$configdir."/charsets";
my %charsets;
my $xmlconfig=$configdir."/gemx.sav";
my $xmldefault=(qq{<?xml version="1.0"?><opt><COMMON NORESTRICTED="0" MAILCHECK="0" MAILPASS="Password" MAILUSR="Login" MAILADDR="foo" MAILINTV="5" MAILSUBJ="1" MAILSRVR="pop.gmx.de" ANTIIDLE="0" REJOIN="15" BEEP="0" BLOCK_QUERIES="0" TRAY="1" COLORPASTE="0" CONFMODE="0" CTCP_SOUND_ALLOW="1" CTCP_SOUND_PLAYIFPLAYING="1" CTCP_SOUND_REQUEST="1" FIXNICK="0" HIDE_REQUESTS="1" HIDE_ALL_REQUESTS="1" PREVIEW="1" SHOW_BAD_NICKCHANGES="1" TIME_HALF_HOURLY="0" TIME_HOURLY="1" TRANSIN="0" TRANSOUT="0" TEMPS="0" WEATHER="0" WETTER="0" COMMAND_CHAR="!" LOGPRIV="1" LOGPUB="1" LANGUAGE="EN" ACTLANG="EN" GENDER="M" IPLOGIN="" IPPASS="" GETMP3="1" TRANSWARNED="0" SEND_UNTRANSLATED="0" MP3ACTIONS_DE="spielt gerade|h&#246;rt gerade|nervt die Nachbarn mit|entspannt bei|singt zu" MP3ACTIONS_EN="is now playing|is listening to|terrorizes the neighbours with|is chilling to|is singing to"/></opt>});
my $gemxconfig;

GemX::ReadConfig();

my $debug=0;


sub GemX::ReadConfig {
	if (-e "$xmlconfig") {
		eval {
			$gemxconfig = $xs->XMLin("$xmlconfig");
		}
	} else {
		$gemxconfig = $xs->XMLin($xmldefault);
	}

	# loading charmaps
	open(CHARMAPS, "$charsets");
	while (<CHARMAPS>) {
		chomp;
		$charsets{$_} = $_;
	}
	close (CHARMAPS);

	$gemxconfig->{COMMON}->{FIGFONT}||="standard";
	if ($^O =~ "MSWin32") {
		$gemxconfig->{COMMON}->{FIGDIR}||="$ENV{APPDATA}\\figlet\\";
	} else {
		$gemxconfig->{COMMON}->{FIGDIR}||="/usr/share/games/figlet/fonts";
	}

# Some settings that were added added later

	# Although they will safely be regarded as unset (0) for checks, we would get
	# "uninitialized" warnings if we don't initalize them as 0 here.
	if (!GemX::GetGlobalSet("MAILCHECK")) { GemX::SetChanSet("MAILCHECK",0,2); }
	if (!GemX::GetGlobalSet("MAILSUBJ")) { GemX::SetChanSet("MAILSUBJ",0,2); }
	if (!GemX::GetGlobalSet("MAILINTV")) { GemX::SetChanSet("MAILINTV",2,2); }
	if (!GemX::GetGlobalSet("MAILUSR")) { GemX::SetChanSet("MAILUSR","Login",2); }
	if (!GemX::GetGlobalSet("MAILPASS")) { GemX::SetChanSet("MAILPASS","Passwort",2); }
	if (!GemX::GetGlobalSet("MAILSRVR")) { GemX::SetChanSet("MAILSRVR","pop.gmx.de",2); }
	if (!GemX::GetGlobalSet("MAILADDR")) { GemX::SetChanSet("MAILADDR","gmx.de",2); }
	if (!GemX::GetGlobalSet("IPPASS")) { GemX::SetChanSet("IPPASS","Password",2); }
	if (!GemX::GetGlobalSet("IPLOGIN")) { GemX::SetChanSet("IPLOGIN","Login",2); }
	if (!GemX::GetGlobalSet("NORESTRICTED")) { GemX::SetChanSet("NORESTRICTED","0",2); }
	if (!GemX::GetGlobalSet("PREFERGOOGLE")) { GemX::SetChanSet("PREFERGOOGLE","0",2); }
	if (!GemX::GetGlobalSet("PREFERONESTEP")) { GemX::SetChanSet("PREFERONESTEP","0",2); }
	if (!GemX::GetGlobalSet("FORCETWOSTEP")) { GemX::SetChanSet("FORCETWOSTEP","0",2); }
	if (!GemX::GetGlobalSet("TWOENGINES")) { GemX::SetChanSet("TWOENGINES","0",2); }
	if (!GemX::GetGlobalSet("AUTOCHINESE")) { GemX::SetChanSet("AUTOCHINESE","0",2); }
	if (!GemX::GetGlobalSet("DEBUG")) { GemX::SetChanSet("DEBUG","0",2); }
	if (!GemX::GetGlobalSet("FARTS")) { GemX::SetChanSet("FARTS","0",2); }

	return 1;
}

sub GemX::WriteConfig {
	open my $fh, '>:encoding(utf8)', $xmlconfig;
	$xs->XMLout($gemxconfig, OutputFile => $fh, NoAttr=>1);
	close $fh;
	return 1;
}

sub GemX::IsChanSet {
	# Get current network and channel plus args
	my $network=lc(Xchat::get_info("network"));
	my $channel=lc($_[0]);
	$channel||=lc(Xchat::get_info("channel"));

	if (exists  $gemxconfig->{ LOCAL }) {
	if (exists  $gemxconfig->{LOCAL}{ NETWORK }) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{ $network }) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{$network}{ CHANNEL }) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}{ $channel }) {
		return 1;
	}
	}
	}
	}
	}
	return 0;
}

sub GemX::IsNetSet {
	# Get current network and channel plus args
	my $network=lc(Xchat::get_info("network"));
	my $channel="*SERVER*";

	if (exists  $gemxconfig->{ LOCAL }) {
	if (exists  $gemxconfig->{LOCAL}{ NETWORK }) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{ $network }) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}) {
	if (exists  $gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}{ $channel }) {
		return 1;
	}
	}
	}
	}
	}
	return 0;
}

sub GemX::GetPlayerActions {
	my $network=lc(Xchat::get_info("network"));
	my $channel=lc(Xchat::get_info("channel"));

	my $raw=$_[0];

	my $actions=GemX::GetChanSet("MP3ACTIONS");

	if (!$actions) {
		if (GemX::GetChanSet("ACTLANG") eq "DE") {
			$actions=GemX::GetChanSet("MP3ACTIONS_DE");
		} else {
			$actions=GemX::GetChanSet("MP3ACTIONS_EN");
		}
	}

	if (!$raw) {
		my @actions=split(/\|/,$actions);
		return (@actions);
	}

	return $actions;

}

sub GemX::GetFixedNick {
	my $network=lc(Xchat::get_info("network"));

#	my $fixed=0;
#	$fixed=GemX::GetNetSet("FIXNICK");
	
	my $nick;

#	if ($fixed) {
		$nick=GemX::GetNetSet("FIXEDNICK");
		if (!$nick) {
			$nick=Xchat::get_prefs("irc_nick1");
		}
#	}
#	Xchat::print("Nick: $nick");
	return $nick;
}

sub GemX::GetEasyJoinTree {
	# Walk through all networks
	my $net;
	my $chan;

	my @nets;

	while (($net = each ( %{ $gemxconfig->{EASYJOIN}{NETWORK} } )) ) {
		# Walk through all channels

		my @chans;
		while (($chan = each ( %{ $gemxconfig->{EASYJOIN}{NETWORK}{$net}{CHANNEL} } )) ) {
			my $easyjoin=$gemxconfig->{EASYJOIN}->{NETWORK}{$net}{CHANNEL}{$chan}{EASYJOIN};
			if ($easyjoin) {
				push(@chans,$chan);
#				Xchat::print("Easyjoin channels for $net: @chans");
			}
		}
		if (@chans) {
			my %newnet;
			my $chans = \@chans;

			%newnet = (
		        	net => $net,
		        	chans => $chans,
			);

			my $newnet   = \%newnet;
			push(@nets,$newnet);
		}
	}

	if (@nets) {
		return @nets;
	} else {
		return undef;
	}
}

sub GemX::RemoveEasyJoin {
	my $net=$_[0];
	my $chan=$_[1];

	delete %{$gemxconfig->{EASYJOIN}{NETWORK}{$net}{CHANNEL}}->{$chan};

#	Xchat::print Dumper($gemxconfig);

	return 1;
}

sub GemX::IsOn {
	my $net=$_[0];
	my $chan=$_[1];
	my $neg=$_[2];

	my @chanlist=Xchat::get_list("channels");

	my $isopen=0;

	for my $n (0 .. ($#chanlist)) {
		if (lc(($chanlist[$n]->{"network"})) eq lc($net)) {
			if (!$chan) {
				if ($neg) {
					return 0;
				} else {
					return 1;
				}
			} elsif (lc(($chanlist[$n]->{"channel"})) eq lc($chan)) {
				if ($neg) {
					return 0;
				} else {
					return 1;
				}
			}
		}
	}

	if ($neg) {
		return 1;
	} else {
		return 0;
	}
}


# Gets the value of a specific channel setting for the current channel
# Asks parents (network, global) for the value if not set for the network
#
# Arguments: <string:setting> - Name of the setting to query
# Returns:   <string:value>   - Setting's value or undef
#
sub GemX::GetChanSet {
	my $network=lc(Xchat::get_info("network"));
	my $channel=lc($_[1]);
	$channel||=lc(Xchat::get_info("channel"));
	my $chanset=uc($_[0]);
	my $return;

	my $local="LOCAL";

	if ($chanset eq "EASYJOIN") {
		if (exists  $gemxconfig->{ EASYJOIN }) {
		if (exists  $gemxconfig->{EASYJOIN}{ NETWORK }) {
		if (exists  $gemxconfig->{EASYJOIN}{NETWORK}{ $network }) {
		if (exists  $gemxconfig->{EASYJOIN}{NETWORK}{$network}{ CHANNEL }) {
		if (exists  $gemxconfig->{EASYJOIN}{NETWORK}{$network}{CHANNEL}{ $channel }) {

			$return=$gemxconfig->{EASYJOIN}->{NETWORK}{$network}{CHANNEL}{$channel}{EASYJOIN};
			if ($return) {
				return $return;
			} else {
				return 0;
			}
		}
		}
		}
		}
		}
	} elsif ($chanset=~ m/FARTED|FARTWORST|FARTBEST|FARTAVG/) {
		if (exists  $gemxconfig->{ FARTS }) {
		if (exists  $gemxconfig->{FARTS}{ CHANNEL }) {
		if (exists  $gemxconfig->{FARTS}{CHANNEL}{ $channel }) {

			$return=$gemxconfig->{FARTS}->{CHANNEL}{$channel}{$chanset};
			if ($return) {
				return $return;
			} else {
				return 0;
			}
		}
		}
		}
	}

#	Xchat::print("$network - $channel - $chanset");

	if (GemX::IsChanSet($channel)) {
		$return=$gemxconfig->{$local}->{NETWORK}{$network}{CHANNEL}{$channel}{$chanset};
		if ($return ne undef) {
			if ($debug) {
				Xchat::emit_print("Generic Message","$gx::info","Returning $return for $chanset in $channel $network");
			}
			return $return;
		}
	}

	$channel="*SERVER*";
	if (GemX::IsNetSet()) {
		$return=$gemxconfig->{$local}{NETWORK}{$network}{CHANNEL}{$channel}{$chanset};
		if ($return ne undef) {
			if ($debug) {
				Xchat::emit_print("Generic Message","$gx::info","Returning $return for $chanset in $channel $network (From Network setting)");
			}
			return $return;
		}
	}

	$return=$gemxconfig->{COMMON}->{$chanset};
	if ($debug) {
		Xchat::emit_print("Generic Message","$gx::info","Returning $return for $chanset in $channel $network (From global setting)");
	}
	return $return;
}



# Gets the value of a specific setting for the current network
# Asks parent (global) for the value if not set for the channel
#
# Arguments: <string:setting> - Name of the setting to query
# Returns:   <string:value>   - Setting's value or undef
#
sub GemX::GetNetSet {
	my $network=lc(Xchat::get_info("network"));
	my $channel="*SERVER*";
	my $chanset=uc($_[0]);
	my $return;

	if (GemX::IsNetSet()) {
		$return=$gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}{$channel}{$chanset};
		if ($return ne undef) {
			return $return;
		}
	}
	$return=$gemxconfig->{COMMON}->{$chanset};
}

# Gets the value of a specific global setting
#
# Arguments: <string:setting> - Name of the setting to query
# Returns:   <string:value>   - Setting's value or undef
#
sub GemX::GetGlobalSet {
	return $gemxconfig->{COMMON}->{uc($_[0])};
}

# Get/Set a channel setting (globally)
#  optionally apply it to all existing channels
# Does not crawl through parents for GETs!
#
# Arguments: (<string:setting>, <*:newvalue>, <int:mode>)
#  setting  = Setting to get/set
#  newvalue = New value for setting
#	      undef -> return current value
#  mode     = 2 - Make/Get global default
#             1 - Make/Get network default
#             0 or undef  - Get/Set for current channel only
#  Note: As of now, modes 2 and 4 only work for setting,
#  getting settings for multiple channels at once is not supported yet
#  Attempts to do so fall back to mode 1 resp. 3
#
#
# Returns: (<int:errlevel>, oldval>)
#
# Errlevel:
#	    -4 - Wrong target, e.g. global-only setting applied on network
#	    -3 - Tried to apply a non-bool value to a bool setting
#	    -2 - Not a legal setting
#	    -1 - Error
#            0 - nothing changed
#            1 - channel/network/global changes made
#           >1 - channel/network/global settings applied x times
# oldval:
#        for all errorlevels but 0 and 1: undef
#        for errorlevels 0 and 1:         old value

sub GemX::SetChanSet {
	# Get current network and channel plus args
	my $network=lc(Xchat::get_info("network"));
	my $chanset=uc("$_[0]");
	my $value=("$_[1]");
	my $mode=uc("$_[2]");
	my $channel=lc("$_[3]");
	$channel||=lc(Xchat::get_info("channel"));
	my $local="LOCAL";

	# Check for settings that can only be used globally
	my $matched=$chanset=~ m/(GENDER|IPLOGIN|IPPASS|BLOCK_QUERIESFIGFONT|ASCIIDIR|FIGDIR|USERINFO|MAILCHECK|MAILPASS|MAILUSR|MAILADDR|MAILINTV|MAILSUBJ|MAILSRVR|AUTOCHINESE|FORCETWOSTEP|PREFERONESTEP|TWOENGINES|PREFERGOOGLE|DEBUG)/;
	if (($matched) && ($mode != 2)) {
		# No such non-global setting -> return with errlevel -4
		return (-4,undef);
	}

	# Check for settings that can only be used per channel
	my $matched=$chanset=~ m/(CHANLANG|EASYJOIN|TRANSIN|TRANSOUT|SEND_UNTRANSLATED|CHARSET|FARTED|FARTBEST|FARTWORST|FARTAVG)/;
	if (($matched) && ($mode != 0)) {
		# No such non-global setting -> return with errlevel -4
		return (-4,undef);
	} elsif (($matched) && ($chanset eq "EASYJOIN")) {
		$local="EASYJOIN";
	} elsif (($matched) && ($chanset=~ m/FARTED|FARTAVG|FARTBEST|FARTWORST/)) {
		$local="FARTS";
	}

	# UpCase values for settings with fixed value choices
	my $matched=$chanset=~ m/(LANGUAGE|CHANLANG|GENDER|ACTLANG|CHARSET)/;
	if ($matched) {
		GemX::UnifyCodepage($value) if ($chanset=~ m/CHARSET/i);

		$value=uc($value);
	}


	my $oldval;
	my $changes=0;

	# Check if setting is legal
	my $matched=$chanset=~ m/(ANTIIDLE|IPLOGIN|IPPASS|LOGPRIV|LOGPUB|BLOCK_QUERIES|WETTER|WEATHER|TEMPS|AUTOREJOIN|BEEP|TRAY|COLORPASTE|CONFMODE|CTCP_SOUND_ALLOW|CTCP_SOUND_PLAYIFPLAYING|CTCP_SOUND_REQUEST|FIXNICK|HIDE_REQUESTS|HIDE_ALL_REQUESTS|PREVIEW|SHOW_BAD_NICKCHANGES|TIME_HALF_HOURLY|TIME_HOURLY|COMMAND_CHAR|LANGUAGE|GETMP3|ACTLANG|GENDER|TRANSOUT|TRANSIN|TRANSWARNED|SEND_UNTRANSLATED|CHANLANG|MP3ACTIONS|MP3ACTIONS_DE|MP3ACTIONS_EN|MP3ACTIONS_FR|ACTLANG|EASYJOIN|ASCIIDIR|FIGFONT|FIGDIR|USERINFO|CREATE|DELETE|MAILCHECK|MAILPASS|MAILUSR|MAILADDR|MAILINTV|MAILSUBJ|MAILSRVR|NORESTRICTED|AUTOCHINESE|FORCETWOSTEP|PREFERONESTEP|TWOENGINES|PREFERGOOGLE|DEBUG|CHARSET|FARTED|FARTWORST|FARTBEST|FARTAVG|FARTS)/;

	if (!$matched) {
		# No such setting -> return with errlevel -2
		return (-2,undef);
	}


	# Setting is legal

	# Check if we are going to set a boolean setting with non-bool value
	$matched=$chanset=~ m/(TRANSOUT|TRANSIN|TRANSWARNED|SEND_UNTRANSLATED|ANTIIDLE|LOGPRIV|LOGPUB|FARTS|WETTER|WEATHER|TEMPS|AUTOREJOIN|AUTOREJOIN_INSTANT|BEEP|TRAY|COLORPASTE|CONFMODE|CTCP_SOUND_ALLOW|CTCP_SOUND_PLAYIFPLAYING|CTCP_SOUND_REQUEST|FIXNICK|HIDE_REQUESTS|HIDE_ALL_REQUESTS|PREVIEW|SHOW_BAD_NICKCHANGES|TIME_HALF_HOURLY|GETMP3|TIME_HOURLY|EASYJOIN|MAILCHECK|MAILSUBJ|NORESTRICTED|AUTOCHINESE|FORCETWOSTEP|PREFERONESTEP|TWOENGINES|PREFERGOOGLE|DEBUG)/;
	if ($matched) {
		my $true=$value=~ s/(YES|ON|TRUE|1)/1/i;
		my $false=$value=~ s/(NO|OFF|FALSE|0)/0/i;

		if ((!$true) && (!$false) && ($value ne undef)) {
			return (-3,undef);
		}
	}

	# Check if we are going to set charset setting a non-supported charset
	$matched=$chanset=~ m/(CHARSET)/;
	if ($matched) {
		my $acharset = lc($value);
		if ((defined $charsets{$acharset}) || ($value eq undef)) {
		} else {
			return (-3,undef);
		}
	}



	# Value appears to be ok or is not defined

#	Xchat::print("Setting $chanset in $channel $network to $value for tree $local");

	# Get/Set global default
	if ($mode == 2) {
		$oldval=$gemxconfig->{COMMON}->{$chanset};
		if (($value ne undef) && ("$value" ne "$oldval")) {
			$gemxconfig->{COMMON}{$chanset}=$value;
			$changes++;
			if ($debug) {
				Xchat::emit_print("Generic Message","$gx::info","Changed global default for $chanset from $oldval to $value");
				Xchat::emit_print("Generic Message","$gx::info","$chanset - $value - Mode: $mode - Changes: $changes");
			}
		} else {
			if ($debug) {
				Xchat::emit_print("Generic Message","$gx::info","Global default for $chanset unchanged at $oldval");
				Xchat::emit_print("Generic Message","$gx::info","$chanset - $value - Mode: $mode - Changes: $changes");
			}
		}
	}

	# Get/Set network default
	if ($mode == 1) {
		$channel="*SERVER*";
	}


	# Get/Set channel setting
	if ($mode == 0) {
		if ($chanset eq "EASYJOIN") {
			if (GemX::GetContextType() == 2) {
				if ($value) {
					$gemxconfig->{$local}->{NETWORK}->{$network}->{CHANNEL}{$channel}{EASYJOIN}=1;
					return 1;
				}
			}
			return 0;
		} elsif (($chanset=~ m/FARTED|FARTWORST|FARTBEST|FARTAVG/) && (!($channel=~ m/\*SERVER\*|\#.*/))) {
			if ($value) {
#				Xchat::print("Setting $chanset in $channel $network to $value for tree $local");
				$gemxconfig->{$local}->{CHANNEL}->{$channel}{$chanset}=$value;
				return 1;
			}
			return 0;
		}
	}

	if (($mode == 0) || ($mode == 1)) {
		if ($chanset eq "CREATE") {
			if ($debug) {
				Xchat::print("About to create an entry for $channel in $network");
			}
			my $blah=$gemxconfig->{LOCAL}->{NETWORK}->{$network}->{CHANNEL}->{$channel}{""};
			return 1;
		} elsif ($chanset eq "DELETE") {
			if ($debug) {
				Xchat::print("About to remove the entry for $channel in $network");
			}
			delete $gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}{ $channel };
			return 1;
		} else {
			if (((GemX::IsNetSet) && ($mode == 1)) || ((GemX::IsChanSet($channel)) && ($mode == 0))) {
				$oldval=$gemxconfig->{$local}->{NETWORK}{$network}{CHANNEL}{$channel}{$chanset};
				if ($value eq undef) {
					delete %{$gemxconfig->{LOCAL}{NETWORK}{$network}{CHANNEL}{$channel}}->{$chanset};
					if ($debug) {
						Xchat::emit_print("Generic Message","$gx::info","Channel setting $chanset in $channel\@$network removed");
						Xchat::emit_print("Generic Message","$gx::info","$chanset - $value - Mode: $mode - Changes: $changes");
					}
				} elsif ($value ne $oldval) {
					$gemxconfig->{$local}->{NETWORK}{$network}{CHANNEL}{$channel}{$chanset}=$value;
					$changes++;
					if ($debug) {
						Xchat::emit_print("Generic Message","$gx::info","Channel setting $chanset in $channel\@$network changed from $oldval to $value");
						Xchat::emit_print("Generic Message","$gx::info","$chanset - $value - Mode: $mode - Changes: $changes");
					}
				}
			}
		}

	}

	if ($debug) {
		Xchat::emit_print("Generic Message","$gx::info","About to return");
		Xchat::emit_print("Generic Message","$gx::info","$chanset - $value - $oldval - Mode: $mode - Changes: $changes");
	}
	return ($changes,undef);
}

sub GemX::GetContextType {
	return Xchat::context_info()->{type};
}

sub GemX::ShowConfig {
	Xchat::print Dumper ($gemxconfig);
	return 1;
}


##################################################
# Converts an amout of time in seconds to an     #
# amount in mins:secs format.                    #
##################################################
sub GemX::ConvertMsecToDhms {
	my $msec=shift(@_);

	my $days=$msec/(1000*60*60*24);
	my $hours=($days - int($days))*24;
	my $mins=($hours - int($hours))*60;
	my $secs=($mins - int($mins))*60;

	$days=int($days);
	$hours=int($hours);
	$mins=int($mins);
	$secs=int($secs);

	if ($hours < 10) {
		$hours="0".$hours;
	}
	if ($mins < 10)	{
		$mins="0".$mins;
	}
	if ($secs < 10)	{
		$secs="0".$secs;
	}

	my $tmp=$days.":".$hours.":".$mins.":".$secs;
	return $tmp;
}

sub GemX::ConvertSecToMin {
	my $insecs=shift(@_);
	my $outmin=int($insecs/60);
	my $outsec=int(($insecs/60 - $outmin)*60);
	if ($outsec < 10) {
		$outsec="0".$outsec;
	}
	return ($outmin,$outsec)
}


#	Sub GemX::Translate
#	Translates any text between two languages using Google Translate
#	or/and Babelfish.
#
#	Will translate a text between ANY two languages that are supported by
#	Google Translate and/or Babelfish, regardless if Google/Babelfish
#	support that language pair. Language pairs not directly supported by
#	Google/Babelfish will be translated in two steps:
#	Source language -> English -> Target language.
#
#	Input:
#		1. Source language (Two digit language code)
#		2. Target language (Two digit language code)
#		3. Text to translate (UTF-8)
#
#	Returns:
#		1. 0 = failed, 1 = success
#		2. Text direction (ltr or rtl) or undef on fail
#		3. Translated text (UTF-8) or undef on fail
#
#	Uses these GemX-settings:
#		PREFERGOOGLE  0 = Use Babelfish, 1 = Use Google Translate
#			      This setting will automatically be overridden
#			      if the selected engine does not support the
#			      source or target language.
#		PREFERONESTEP 0 = Use two steps, even if other engine could do it in one step,
#			      1 = Switch engine if other one can do the translation in one step
#		FORCETWOSTEP  0 = Use one step if possible and/or requested via PREFERONESTEP
#			      1 = Always use two steps, even if the current one could do it
#			          one step (Gives better results if a mixture of any language
#				  plus English is used).
#		TWOENGINES    0 = In two-step translations, use the same engine for both steps, if possible,
#			      1 = In two-step translations, use the prefered engine for supported steps
#		AUTOCHINESE   0 = Stick with type of Chinese from parameter,
#			       1 = Autodetect style of chinese source text)
#			      Does not apply to translations between both Chinese variants
sub GemX::Translate {
	my $from=lc($_[0]);	# Source language
	my $to=lc($_[1]);	# Target language
	my $text=$_[2];		# Text to be translated

	# 0 = one-step translation, 1 = translate via other language
	# In the beginning, assume we need two steps 
	my $via=1;		

	# 1 = Google, 0 = Babelfish
	my $google=GemX::GetGlobalSet("PREFERGOOGLE");

	Xchat::emit_print("Generic Message",$gx::info,"Translate called with From ".$from." - To ".$to." - Text: ".$text." - Google selected = ".$google) if (GemX::GetGlobalSet("DEBUG"));

	if ($to eq $from) {
		return (0,undef,undef);
	}

	my $success;

#	Valid langpairs:

#	Both engines:
#	de|fr  Deutsch nach Franzsisch
#	de|en  Deutsch nach Englisch
#	fr|de  Franzsisch nach Deutsch
#	en|de  Englisch nach Deutsch
#	en|fr  Englisch nach Franzsisch
#	en|el  Englisch nach Griechisch
#	en|it  Englisch nach Italienisch
#	en|ja  Englisch nach Japanisch
#	en|ko  Englisch nach Koreanisch
#	en|nl  Englisch nach Niederlndisch
#	en|pt  Englisch nach Portugiesisch
#	en|ru  Englisch nach Russisch
#	en|es  Englisch nach Spanisch
#	el|en  Griechisch nach Englisch
#	fr|en  Franzsisch nach Englisch
#	it|en  Italienisch nach Englisch
#	ja|en  Japanisch nach Englisch
#	ko|en  Koreanisch nach Englisch
#	nl|en  Niederlndisch nach Englisch
#	pt|en  Portugiesisch nach Englisch
#	ru|en  Russisch nach Englisch
#	es|en  Spanisch nach Englisch

#	Chinese targets
	# Babelfish
	#	en_zh  Englisch nach Chinesisch (vereinfacht)
	#	en_zt  Englisch nach Chinesisch (traditionell)

	# Google
	#	en|zh-TW  Englisch nach Chinesisch (traditionell)
	#	en|zh-CN  Englisch nach Chinesisch (vereinfacht)

#	Babelfish only:
#	fr_de  Franzsisch nach Deutsch
#	fr_el  Franzsisch nach Griechisch
#	fr_it  Franzsisch nach Italienisch
#	fr_nl  Franzsisch nach Niederlndisch
#	fr_pt  Franzsisch nach Portugiesisch
#	fr_es  Franzsisch nach Spanisch
#	de_fr  Deutsch nach Franzsisch
#	el_fr  Griechisch nach Franzsisch
#	it_fr  Italienisch nach Franzsisch
#	nl_fr  Niederlndisch nach Franzsisch
#	pt_fr  Portugiesisch nach Franzsisch
#	es_fr  Spanisch nach Franzsisch
#	zh_en  Chinesisch (vereinfacht) nach Englisch
#	zt_en  Chinesisch (traditionell) nach Englisch

#	Google only:
#	ar|en  Arabisch nach Englisch
#	en|ar  Englisch nach Arabisch
#	zh-TW|zh-CN Chinesisch (traditionell nach vereinfacht)
#	zh-CN|zh-TW  Chinesisch (vereinfacht nach traditionell)
#	zh|en  Chinesisch (automatisch) nach Englisch

	# Check One
	# If PREFERONESTEP is active, check if we need to autoswitch the
	# engine to avoid a two-step translation.
	# FORCETWOSTEP takes precedence over PREFERONESTEP

	if ((GemX::GetGlobalSet("PREFERONESTEP")) && (!GemX::GetGlobalSet("FORCETWOSTEP"))) {
		if (!$google) {
		# Babelfish selected
			# At the moment, Google Translate can not translate any
			# langpairs directly that Babelfish can not.
		} else {
		# Google selected
			# Only Babelfish can translate French directly to
			# Dutch, Italien, Greek, Portuguese or Spanish
			if (($from eq "fr") && (($to eq "nl") || ($to eq "el") || ($to eq "it") || ($to eq "pt") || ($to eq "es"))) {
				$google=0;
			}

			# Only Babelfish can translate
			# Dutch, Italien, Greek, Portuguese or Spanish
			# directly to French
			if (($to eq "fr") && (($from eq "nl") || ($from eq "el") || ($from eq "it") || ($from eq "pt") || ($from eq "es"))) {
				$google=0;
			}
		}
	}

	Xchat::emit_print("Generic Message",$gx::info,"Translate after PREFERONESTEP: From ".$from." - To ".$to." - Text: ".$text." - Google selected = ".$google) if (GemX::GetGlobalSet("DEBUG"));

	# Check Two
	# Two-step necessary?

	# Languages that either engine can translate to English
	if (($from eq "ar") || ($from eq "ja") || ($from eq "ko") || ($from eq "ru") || ($from eq "zh") || ($from eq "zt") || ($from eq "nl") || ($from eq "de") || ($from eq "el") || ($from eq "it") || ($from eq "pt") || ($from eq "es")) {
		if ($to eq "en") {
			$via=0;
		}
	}

	if (($to eq "ar") || ($to eq "ja") || ($to eq "ko") || ($to eq "ru") || ($to eq "zh") || ($to eq "zt") || ($to eq "nl") || ($to eq "de") || ($to eq "el") || ($to eq "it") || ($to eq "pt") || ($to eq "es")) {
		if ($from eq "en") {
			$via=0;
		}
	}

	# From French (Babelfish supports more target languages for it)
	if ($from eq "fr") {
		if (!$google) {
			if (($to eq "en") || ($to eq "nl") || ($to eq "de") || ($to eq "el") || ($to eq "it") || ($to eq "pt") || ($to eq "es")) {
				$via=0;
			}
		} else {
			if (($to eq "en") ||($to eq "de")) {
				$via=0;
			}
		}
	}

	# To French (Babelfish supports more source languages for it)
	if ($to eq "fr") {
		if (!$google) {
			if (($from eq "en") || ($from eq "nl") || ($from eq "de") || ($from eq "el") || ($from eq "it") || ($from eq "pt") || ($from eq "es")) {
				$via=0;
			}
		} else {
			if (($from eq "en") || ($from eq "de")) {
				$via=0;
			}
		}
	}

	Xchat::emit_print("Generic Message",$gx::info,"Translate after two-step necessarity check: From ".$from." - To ".$to." - Text: ".$text." - Google selected = ".$google." - Two-step: ".$via) if (GemX::GetGlobalSet("DEBUG"));

	# Check Three
	# Automatically change the translation engine if the chosen one
	# cannot translate the langpair.

	if ($from eq "ar") {
		# Only Google Translate can translate from Arabic
		$google=1;
	}
	if (($from eq "en") && ($to eq "ar")) {
		# Only Google Translate can translate to Arabic
		# We can only translate to Arabic from English, every other pair
		# requires a two-step translation, by limiting the from language
		# to English, we allow Babelfish (If being the prefered engine)
		# to do the first step. 
		$google=1;
	}

	if (($from eq "zh") || ($from eq "zt")) {
		# Translating between Chinese?
		if (($to eq "zh") || ($to eq "zt")) {
			# Yes.
			# Only Google can do this:
			$google=1;
			$via=0;
			# Google uses different ids for Chinese:
			$from=~s /zt/zh-TW/i;
			$from=~s /zh/zh-CN/i;
		} else {
			# No. Google preferred or auto-detect source chinese?
			if (($google) || (GemX::GetGlobalSet("AUTOCHINESE"))) {
				# Only google can do this:
				$google=1;
				# Google uses different ids for Chinese:
				$from="zh";	# Chinese auto
			}
		}
	}

	Xchat::emit_print("Generic Message",$gx::info,"Translate after engine compatibility check: From ".$from." - To ".$to." - Text: ".$text." - Google selected = ".$google." - Two-step: ".$via) if (GemX::GetGlobalSet("DEBUG"));

	# Check Four
	# Switch back to two-step if we aren't translating from or to English already
	# and FORCETWOSTEP is enabled

	if (($to ne "en") && ($from ne "en") && (GemX::GetGlobalSet("FORCETWOSTEP"))) {
		$via=1;
	}


	# Check Five
	# Correct some slang or typical typographical errors to support
	# the engine in translation.

	# French
	if ($from eq "fr") {
		$text=~ s/\bbonenuit\b/bonne nuit/gi;
		$text=~ s/\b(mut|mout)\b/moût/gi;
		$text=~ s/\bdsl\b/désolé/gi;
		$text=~ s/\bqq\b/quelque/gi;
		$text=~ s/\bqqn\b/quelqu'un/gi;
		$text=~ s/\bouaip\b/oui/gi;
		$text=~ s/\blut\b/salut/gi;
		$text=~ s/\bca\b/ça/gi;
		$text=~ s/\by'a\b/il y a/gi;
		$text=~ s/\bcad\b/c'est-à-dire/gi;
		$text=~ s/\bt'es\b/tu es/gi;
	}

	# German
	if ($from eq "de") {
		# Un-Stretching umlauts (Like in s = sweeeeeeet)
		$text=~ s/ö+/ö/gi;
		$text=~ s/ä+/ä/gi;
		$text=~ s/ü+/ü/gi;

		# (Chat) slang and acronyms
		$text=~ s/\bnabend\b/guten Abend/gi;
		$text=~ s/\bgn8\b/gute Nacht/gi;
		$text=~ s/\b8ung\b/Achtung/gi;
		$text=~ s/\bnix\b/nichts/gi;
		$text=~ s/\btach\b/Tag/gi;
		$text=~ s/\bfickön\b/ficken/gi;
		$text=~ s/\brollig\b/geil/gi;
		$text=~ s/\brattig\b/geil/gi;
		$text=~ s/\bisses\b/ist es/gi;
		$text=~ s/\banneres\b/anderes/gi;
		$text=~ s/\bfürn\b/für ein/gi;
		$text=~ s/\bübers\b/über das/gi;
		$text=~ s/\blaber/rede/gi;
		$text=~ s/\bnu/nun/gi;
		$text=~ s/\bhaste/hast Du/gi;
		$text=~ s/\bzuckersüß\b/süß wie Zucker/gi;
		$text=~ s/\bwe\b/Wochenende/gi;
		$text=~ s/\brotzfrech\b/sehr frech/gi;
		$text=~ s/\bnee+\b/nein/gi;
		$text=~ s/\bne\b/eine/gi;
		$text=~ s/\bnen\b/einen/gi;
		$text=~ s/\bwasn\b/was ist denn/gi;
		$text=~ s/\bkl\./kleinen/gi;
		$text=~ s/\bfiggn\b/ficken/gi;
		$text=~ s/\bfiggen\b/ficken/gi;

		# Typical mistakes
		$text=~ s/\bvotze\b/Fotze/gi;

		# Words google doesn't know
		$text=~ s/\bpoppen\b/ficken/gi;
		$text=~ s/\bmöpse+?\b/Titten/gi;	# Tits are more likely than pugs
		$text=~ s/\bwill\b/möchte/gi;		# "will"

		# "Cutified" words
		$text=~ s/\bFötzchen\b/kleine Fotze/gi;
		$text=~ s/\bSälchen\b/kleiner Saal/gi;

		# Dialect
		$text=~ s/\bSers\b/Servus/gi;
		$text=~ s/\bDere\b/Habe die Ehre/gi;
	}

	$text = encode("utf8", $text);

#	Xchat::print("Going to translate $text From: $from To: $to Via: $via Google: $google");

	my $browser = LWP::UserAgent->new;

	$browser->timeout(60);
	$browser->agent('Mozilla/5.0 (Windows; U; Windows NT 5.1; de; rv:1.8.1.11) Gecko/2007112718 Firefox/2.0.0.11');
	$browser->default_header('Accept-Charset' => "utf-8");

	my $url;
	my %form;

	my $langpair;
	my $response;

	my $translated;
	my $dir;

#	Engine interaction.

	if ($google) {
#	Google Translate
		# Google uses different ids for Chinese:
		$to=~s /zh/zh-CN/i;
		$to=~s /zt/zh-TW/i;

#		Xchat::print("Google");
		if ($via) {
			$langpair=$from."|"."en";
		} else {
			$langpair=$from."|".$to;
		}

		$url = "http://www.google.com/translate_t";

		%form = (
			'text' => $text,
			'langpair' => $langpair,
			'hl' => "de",
			'ie' => "UTF-8",
			'oe' => "UTF-8",
			'submit' => "Übersetzen"
		);
		Xchat::emit_print("Generic Message",$gx::info,"Translate using Google - Langpair: ".$langpair." - Text: ".$text) if (GemX::GetGlobalSet("DEBUG"));
		$response = $browser->post( $url, \%form );
	}

	Xchat::print Dumper ($response) if (GemX::GetGlobalSet("DEBUG"));

	$to=~s /zh-TW/zt/i;
	$to=~s /zh-CN/zh/i;
	$from=~s /zh-TW/zt/i;
	$from=~s /zh-CN/zh/i;

	Xchat::emit_print("Generic Message",$gx::info,"Success: ".$response->is_success." Redirect: ".$response->is_redirect) if (GemX::GetGlobalSet("DEBUG"));

	if ((!$google) || ((!$response->is_success) || ($response->is_redirect))) {
#	Babelfish
		# Is it a fallback from Google?
		if ($google) {
			# We can not fall back to Babelfish if it doesn't
			# support the requested translation:
			if ($langpair eq "zh|zt") {
				return (0,undef,undef);
			}
			if ($langpair eq "zt|zh") {
				return (0,undef,undef);
			}
			if ($langpair=~ m/ar/i) {
				return (0,undef,undef);
			}
			$google=0;
			Xchat::emit_print("Generic Message",$gx::info,"Translate using Babelfish Fallback - Langpair: ".$langpair." - Text: ".$text) if (GemX::GetGlobalSet("DEBUG"));
		} else {
			Xchat::emit_print("Generic Message",$gx::info,"Translate using Babelfish - Langpair: ".$langpair." - Text: ".$text) if (GemX::GetGlobalSet("DEBUG"));
		}
		if ($via) {
			$langpair=$from."_en";
		} else {
			$langpair=$from."_".$to;
		}

		$url = "http://babelfish.altavista.com/tr";

		%form = (
			'action' => qq{http\:\/\/babelfish\.altavista\.com\/tr},
			'trtext' => $text,
			'lp' => $langpair,
			'Submit' => qq{Translate},
		);
		Xchat::emit_print("Generic Message",$gx::info,"Translate using Babelfish - Langpair: ".$langpair." - Text: ".$text) if (GemX::GetGlobalSet("DEBUG"));
		$response = $browser->post( $url, \%form );
  	}


#	Xchat::print Dumper ($response);


	if (!$response->is_success) {
		return (0,undef,undef);
	}

	my $content=$response->content;

#	Xchat::print Dumper ($content);
	

	if ($google) {
#	Google Translate
		$content=~m /<div id=result_box dir=(.*?)>(.*?)<\/div>/;
		$dir=$1;
		$translated = decode("utf8", $2);
	} else {
#	Babelfish
		$content=~s /.*<\!\-\- Target text \(content\) \-\->//gs;
		$content=~s /<\!\-\- end\: Target text \(content\).*//gs;
		$content=~m /<div.*?>(.*?)<\/div.*?>/gs;
		$translated = decode("utf8", $1);
		$translated=~ s/\n/ /g;
		$translated=~ s/\r/ /g;
		# Babelfish doesn't report the text direction, but it only
		# supports languages using ltr
		$dir="ltr";
	}

	$translated=~ s/&#39;/'/g;
	$translated=~ s/\&gt;/\>/g;
	$translated=~ s/\&lt;/\</g;
	$translated=~ s/\&amp;/\&/g;
	$translated=~ s/\&quot;/\"/g;

	if (($via) && ($translated)) {
		($success,$dir,$translated)=GemX::Translate("en",$to,$translated);
		if (!$success) {
			return (0,undef,undef);
		}
	}

	if ($translated) {
		return (1,$dir,$translated);
	} else {
		return (0,undef,undef);
	}
}

sub GemX::IsTabOpen {
	my $channel=$_[0];
	my $network=$_[1];

	$channel||=Xchat::get_info("channel");
	$network||=Xchat::get_info("network");
	
	my @chanlist=Xchat::get_list("channels");

	my $isopen=0;

	for my $n (0 .. ($#chanlist)) {
#		Xchat::print("Testing ".$chanlist[$n]->{"channel"}." in ".$chanlist[$n]->{"network"});
		if ((lc($chanlist[$n]->{"channel"}) eq lc($channel)) && (lc($chanlist[$n]->{"network"}) eq lc($network))) {
			return $chanlist[$n]->{"context"};
		}
	}
	return 0;
}

sub GemX::GetPOP3 {
	my $i;
	my @mails;

	my $pop = new Mail::POP3Client( USER     => GemX::GetGlobalSet("MAILUSR"),
					PASSWORD => GemX::GetGlobalSet("MAILPASS"),
					HOST     => GemX::GetGlobalSet("MAILSRVR") );


	my $count=$pop->Count;

	return -1 if ($count == -1);	# An error occured

	# Check if there are any messages at all
	if ($count) {
		# Gather unique id of the last message on the server
		my $uidl=$pop->Uidl($count);	
		my @uidl=split(/ /,"$uidl",2);
		$uidl=@uidl[1];			
		chomp $uidl;			

		# Return with 0 (No new mails) if the unique ID of the last message
		# is the same as during the previous check
		if ("$uidl" eq "$last_uidl") {
			$pop->Close;
			return 0;
		}

		# Store the new last message's unique ID as last checked UID
		$last_uidl=$uidl;

		# Create new arrays for Froms and Subjects
		my @froms;
		my @subjects;

		# Populate the arrays
		for ($i = 1; $i <= $count; $i++) {
			foreach ( $pop->Head( $i ) ) {
				/^From:\s+(.+)/i and push(@froms,$1);
				/^Subject:\s+(.+)/i and push(@subjects,$1);
			}
		}

		# Close the connection
		$pop->Close;

		# Convert the two arrays into one array of hashes
		for ($i = 0; $i <= $#froms; $i++) {
			my %mail;
			%mail = (
	        		from => @froms[$i],
			        subj => @subjects[$i],
			);
			my $mail   = \%mail;
			push(@mails,$mail);
		}

		# Return the array of hashes
		return (@mails);

	} else {
		# No mails = no new mails
		return 0;
	}
}

######################################################
# GemX::GetSoundDir();                               #
# Returns the sounddir, properly formatted and       #
# converts relative sound_dir to absolute sound_dir  #
# beginning with user's config dir                   #
######################################################
sub GemX::GetSoundDir {
	my $snddir=Xchat::get_prefs("sound_dir");
	my $cfgdir=Xchat::get_info("xchatdir");
	$snddir=~ s/^\./$cfgdir/;
	$snddir=~ s/\//\\/g if ($^O =~ "MSWin32");
	return $snddir;
}


sub GemX::UnifyCodepage {
	my $charset=$_[0];

	# Ancient charsets
	# Still being used in environments where some machines might be
	# running DOS or OS/2 (incl. eComStation) ones

	# Original 437 (Hardware codepage)
	$charset=~ s/(cp437|ibm437|csibm437|oem437|437|cspc8codepage437)/cp437/i;

	# Multilingual 850
	$charset=~ s/(cp850|ibm850|csibm850|oem850|850|cspc850multilingual)/cp850/i;

	# Multilingual 858/8501 (850 minus dotless-i plus Euro)
	# Used by OS/2 and PC-DOS 7.x with y2k-Fix as well as PC-DOS 2000
	# although both refer to it as CP850
	$charset=~ s/(cp858|ibm858|csibm858|oem858|858|cspc858multilingual)/cp858/i;
	$charset=~ s/(cp8501|ibm8501|csibm8501|oem8501|8501|cspc8501multilingual)/cp858/i;

	# Cyrillic 866
	$charset=~ s/(cp866|ibm866|csibm866|oem866|866|cspc866)/cp866/i;

	# 737
	$charset=~ s/(cp737|ibm737|csibm737|oem737|737|cspc737)/cp737/i;

	# Baltic 775
	$charset=~ s/(cp775|ibm775|csibm775|oem775|775|cspc775baltic)/cp775/i;

	# 852
	$charset=~ s/(cp852|ibm852|csibm852|oem852|852|cspc852)/cp852/i;

	# 853
	$charset=~ s/(cp853|ibm853|csibm853|oem853|853|cspc853)/cp853/i;

	# 855
	$charset=~ s/(cp855|ibm855|csibm855|oem855|855|cspc855)/cp855/i;

	# 857
	$charset=~ s/(cp857|ibm857|csibm857|oem857|857|cspc857)/cp857/i;

	# 860
	$charset=~ s/(cp860|ibm860|csibm860|oem860|860|cspc860)/cp860/i;

	# 861 (Icelandic)
	$charset=~ s/(cp861|ibm861|csibm861|oem861|861|cspc861|cp-is)/cp861/i;

	# 862 (Hebrew)
	$charset=~ s/(cp862|ibm862|csibm862|oem862|862|cspc862|cspc862latinhebrew)/cp862/i;

	# 863
	$charset=~ s/(cp863|ibm863|csibm863|oem863|863|cspc863)/cp863/i;

	# 864
	$charset=~ s/(cp864|ibm864|csibm864|oem864|864|cspc864)/cp864/i;

	# 865
	$charset=~ s/(cp865|ibm865|csibm865|oem865|865|cspc865)/cp865/i;

	# 869 (Greek)
	$charset=~ s/(cp869|ibm869|csibm869|oem869|869|cspc869|cp-gr)/cp869/i;

	# cp1125


	#
	# ISO 8859-x series
	#

	# ISO-8859-1 Latin-1 (Western) aka Codepage 819
	$charset=~ s/(cp819|ibm819|csibm819|oem819|819)/iso-8859-1/i;
	$charset=~ s/(iso(-|_|)8859-1(:1987|)|iso-ir-100|csisolatin1|l1|latin1)/iso-8859-1/i;

	# ISO-8859-2 Latin-2 (Eastern)
	$charset=~ s/(iso(-|_|)8859-2(:1987|)|iso-ir-101|csisolatin2|l2|latin2)/iso-8859-2/i;

	# ISO-8859-3 Latin-3 (South European, incl. Turkish, Maltese, Esperanto)
	$charset=~ s/(iso(-|_|)8859-3(:1988|)|iso-ir-109|csisolatin3|l3|latin3)/iso-8859-3/i;

	# ISO-8859-4 Latin-4 (Baltic)
	$charset=~ s/(iso(-|_|)8859-4(:1988|)|iso-ir-110|csisolatin4|l4|latin4)/iso-8859-4/i;

	# ISO-8859-5 (Cyrillic)
	$charset=~ s/(iso(-|_|)8859-5(:1988|)|iso-ir-144|csisolatincyrillic|cyrillic)/iso-8859-5/i;

	# ISO-8859-6 (Arabic)
	$charset=~ s/(iso(-|_|)8859-6(:1987|)|iso-ir-127|csisolatinarabic|arabic|asmo-708|ecma-114)/iso-8859-6/i;

	# ISO-8859-7 (Greek)
	$charset=~ s/(iso(-|_|)8859-7(:1987|)|iso-ir-126|csisolatingreek|greek|greek8|ecma-118|elot_928)/iso-8859-7/i;

	# ISO-8859-8 (Hebrew)
	$charset=~ s/(iso(-|_|)8859-8(:1988|)|iso-ir-138|csisolatinhebrew|hebrew)/iso-8859-8/i;

	# ISO-8859-9 Latin-5 (Turkish) replaces Latin-3
	$charset=~ s/(iso(-|_|)8859-9(:1989|)|iso-ir-148|csisolatin5|l5|latin5)/iso-8859-9/i;

	# ISO-8859-10 Latin-6 (Nordic) replaces Latin-4
	$charset=~ s/(iso(-|_|)8859-10(:1992|)|iso-ir-157|csisolatin6|l6|latin6)/iso-8859-10/i;

	# ISO-8859-13 Latin-7 (Baltic) replaces Latin-4 and Latin-6
	$charset=~ s/(iso(-|_|)8859-13|iso-ir-179|csisolatin7|l7|latin7)/iso-8859-13/i;

	# ISO-8859-14 Latin-8 (Celtic)
	$charset=~ s/(iso(-|_|)8859-14(:1998|)|iso-ir-199|csisolatin8|l8|latin8|iso-celtic)/iso-8859-14/i;

	# ISO-8859-15 Latin-9 (Western) replaces Latin-1
	$charset=~ s/(iso(-|_|)8859-15(:1998|)|iso-ir-203|csisolatin9|l9|latin9)/iso-8859-15/i;

	# ISO-8859-16 Latin-10 (South Eastern Europe)
	$charset=~ s/(iso(-|_|)8859-16(:2000|)|iso-ir-226|csisolatin10|l10|latin10)/iso-8859-16/i;

	#
	# Windows Codepages
	#

	# Windows CP1252 (Eastern)
	$charset=~ s/(cp1250|ms-ee|windows-1250)/cp1250/i;

	# Windows CP1251 (Cyrillic)
	$charset=~ s/(cp1251|ms-cyrl|windows-1251)/cp1251/i;

	# Windows CP1252 (Western) compatible superset of Latin-1
	$charset=~ s/(cp1252|ms-ansi|windows-1252)/cp1252/i;

	# Windows CP1253 (Greek)
	$charset=~ s/(cp1253|ms-greek|windows-1253)/cp1253/i;

	# Windows CP1254 (Turkish)
	$charset=~ s/(cp1254|ms-turk|windows-1254)/cp1254/i;

	# Windows CP1255 (Hebrew)
	$charset=~ s/(cp1255|ms-hebr|windows-1255)/cp1255/i;

	# Windows CP1256 (Arabic)
	$charset=~ s/(cp1256|ms-arab|windows-1256)/cp1256/i;

	# Windows CP1257 (Baltic)
	$charset=~ s/(cp1257|winbaltrim|windows-1257)/cp1257/i;

	# Windows CP1258 ()
	$charset=~ s/(cp1258|windows-1258)/cp1258/i;

	#
	# MacOS codepages (pre-MacOS)
	#

	# MacRoman
	$charset=~ s/(mac|macintosh|macroman|csmacintosh)/macroman/i;

	# maccentraleurope
	# maciceland
	# maccroatian
	# macromania
	# maccyrillic
	# macukraine
	# macgreek
	# macturkish
	# machebrew
	# macarabic
	# macthai

	# Plain ASCII (7-Bit subset of all of the above) aka Codepage 367
	$charset=~ s/(ansi_x3.4-1968|ansi_x3.4-1986|ascii|cp367|ibm367|iso-ir-6|iso646-us|iso_646.irv:1991|us|us-ascii|csascii)/cp367/i;

	return $charset;
}

sub GemX::rot13 {
	my $text=$_[0];
	my $return;

	while ($text =~ m/./g) {
		if (ord($&) > 77) {
			$return=$return.chr(ord($&)-13);
		} else {
			$return=$return.chr(ord($&)+13);
		}
	}
	return $return;
}

sub GemX::reverse {
	my $text=$_[0];
	my $return;

	my @chars=split(//, $text);

	while (@chars) {
		my $n=pop(@chars);
		$return=$return.$n;
	}

	return $return;
}

sub GemX::encode {
	my $text=$_[0];
	my $return;

	my @chars=split(//, $text);

	while (@chars) {
		my $n=shift(@chars);
		$return=$return.chr((ord($n) >> 4) + 65);
		$return=$return.chr((ord($n) & 15) + 65);
	}

	return $return;
}

sub GemX::decode {
	my $text=$_[0];
	my $return;

	my @chars=split(//, $text);

	while (@chars) {
		$return=$return.chr( ( ((ord(shift(@chars))) - 65) << 4 ) | ( ((ord(shift(@chars))) - 65)) );
	}

	return $return;
}

