#!/usr/bin/perl

# This script portion is Win32 specific.
# It will only work on Win32 with ActivePerl or 100% compatible perl
# interpreter installed.

use strict;
use File::Spec;
use File::Copy;
use Win32::GUI;
use Win32;
use Win32::Registry;
use Win32::Process;
use Win32::API;
use POSIX;
use POSIX qw(strftime);
use POSIX qw(:sys_wait_h _exit);
use LWP::Simple 'get';
use Encode;
use utf8; 
use threads;
use threads::shared;

use Win32::MCI::Basic;
use Win32::FileOp;
#use Win32::FileOp qw(ShellExecute);
use Config::IniFiles;


my $mp3file : shared;
my $httpqhost='127.0.0.1';
my $httpqport='4800';
my $httpqpass='pass';
$gx::playing = 0;


Xchat::register("GemX: Win32 specific functions and workarounds",$gx::ver,undef,\&chksnd);

Xchat::hook_command("PIZZA", "pizza");
Xchat::hook_command("VGIRLHD", "vgirlhd");
Xchat::hook_command("EMULE", "get_emule_path");
Xchat::hook_command("AMP", "amp");
Xchat::hook_command("PM123", "amp");
Xchat::hook_command("PM", "amp");
Xchat::hook_command("WMP", "wmp");

# Hooks for sound functionality

Xchat::hook_command("SOUND", "sound");
Xchat::hook_command("ASCII", "ascii");
Xchat::hook_print("DCC RECV Complete", "dcc_recv_complete"); 
Xchat::hook_print("DCC RECV Complete", "complete_handler"); 

Xchat::hook_command("FIGFONT", "figfont");
Xchat::hook_command("LOG", "open_log");
Xchat::hook_command("DATA", "open_data");
Xchat::hook_command("PIC", "open_pic");
Xchat::hook_command("FPIC", "fetch_pics");
Xchat::hook_command("DCCIN", "open_dcc");

Xchat::hook_command("FXPFXP", "fxpfxped");
Xchat::hook_command("FXPGEUPPT", "fxpgeuppt");
Xchat::hook_command("FXPGESAUGT", "fxpgesaugt");
Xchat::hook_command("FXPALLE", "fxpalle");

get_sysinfo();

sub get_sysinfo {
	my @wver=Win32::GetOSVersion;

	$gx::sysinfo::system="Windows";
	$gx::sysinfo::system="Windows NT" if ($wver[4] eq "2");

	$gx::sysinfo::version="$wver[1].$wver[2]";

	$gx::sysinfo::vstring=get_win_version();

	$gx::sysinfo::arch="i386";

	$gx::sysinfo::clock="";

	if ($gx::sysinfo::system eq "Windows NT") {
		my $clock=get_cpu_clock();
		$gx::sysinfo::clock=sprintf("%.2fMHz", $clock);
		$gx::sysinfo::clock=sprintf("%.2fGHz", ($clock / 1000)) if ($clock > 999);
	}

	$gx::sysinfo::cpustring=get_chip_arch();

	return 1;
}

sub get_cpu_clock {
	my $hkey;
	$::HKEY_LOCAL_MACHINE->Open("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",$hkey) or die "Can't open registry";

	my ($type, $value);
	
	$hkey->QueryValueEx("\~MHz", $type, $value) or die "No CPU: $^E";

	return $value;
}


sub get_win_version {
	my $os='';
	my @wver=Win32::GetOSVersion;

	if ($wver[4] eq "1") {
		if ($wver[1] eq "4") {
			$os = "Windows 95" if $wver[2] eq "00";
			$os = "Windows 98" if $wver[2] eq "10";
			$os = "Windows Millenium" if $wver[2] eq "90";
			}
	} elsif ($wver[4] eq "2") {
		if ($wver[1].$wver[2] eq "40") {
			$os = "Windows NT 4.0";
		} elsif ($wver[1].$wver[2] eq "50") {
			$os = "Windows 2000";
		} elsif ($wver[1].$wver[2] eq "51") {
			$os = "Windows XP";
		} elsif ($wver[1].$wver[2] eq "52") {
			$os = "Windows XP 2003";
		} elsif ($wver[1].$wver[2] eq "53") {
			$os = "Windows XP 2004";
		} elsif ($wver[1].$wver[2] eq "60") {
			$os = "Windows Vista";
		}
	}
	return $os;
}



sub get_chip_arch {
	my $hkey;
	$::HKEY_LOCAL_MACHINE->Open("HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",$hkey) or die "Can't open registry";

	my ($type, $value, $vendor, $arch, $speed, $model);
	
#	if ("$gx::sysinfo::system" eq "Windows NT") {
#		$hkey->QueryValueEx("ProcessorNameString", $type, $value) or die "No CPU: $^E";
#	} else {
#		$hkey->QueryValueEx("Identifier", $type, $value);
#	}

	$hkey->QueryValueEx("ProcessorNameString", $type, $value);
	if ($value eq "") {
		if ("$gx::sysinfo::system" eq "Windows NT") {
			$hkey->QueryValueEx("Identifier", $type, $arch);
			$hkey->QueryValueEx("VendorIdentifier", $type, $vendor);

			if ($vendor eq "GenuineIntel") {
				$value=("Intel Processor");

				my $search=("x86 Family 4 Model 0");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX25/DX33");
				}
				my $search=("x86 Family 4 Model 1");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX50");
				}
				my $search=("x86 Family 4 Model 2");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486SX");
				}
				my $search=("x86 Family 4 Model 3");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX2 w/o Write-Back");
				}
				my $search=("x86 Family 4 Model 4");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486SL");
				}
				my $search=("x86 Family 4 Model 5");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486SX2");
				}
				my $search=("x86 Family 4 Model 7");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX2 w/ Write-Back");
				}
				my $search=("x86 Family 4 Model 8");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX4 w/o Write-Back");
				}
				my $search=("x86 Family 4 Model 9");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel 486DX4 w/ Write-Back");
				}
				my $search=("x86 Family 5");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium");
				}
				my $search=("x86 Family 6 Model 0");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium");
				}
				my $search=("x86 Family 6 Model 1");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium");
				}
				my $search=("x86 Family 6 Model 3");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-II");
				}
				my $search=("x86 Family 6 Model 5");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-II");
				}
				my $search=("x86 Family 6 Model 6");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-II");
				}
				my $search=("x86 Family 6 Model 7");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-III");
				}
				my $search=("x86 Family 6 Model 8");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-III");
				}
				my $search=("x86 Family 6 Model 9");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-M");
				}
				my $search=("x86 Family 6 Model A");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-III");
				}
				my $search=("x86 Family 6 Model B");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-III");
				}
				my $search=("x86 Family 6 Model D");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("Intel Pentium-M");
				}
			$value=("$value $gx::sysinfo::clock");
			} elsif ($vendor eq "AuthenticAMD") {
				$value=("AMD Processor");

				my $search=("x86 Family 4 Model 3");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 486DX2 w/o Write-Back");
				}
				my $search=("x86 Family 4 Model 7");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 486DX2 w/ Write-Back");
				}
				my $search=("x86 Family 4 Model 8");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 486DX4 w/o Write-Back");
				}
				my $search=("x86 Family 4 Model 9");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 486DX4 w/ Write-Back");
				}
				my $search=("x86 Family 4 Model E");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 5x86 w/o Write-Back");
				}
				my $search=("x86 Family 4 Model 9");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD 5x86 w/ Write-Back");
				}
				my $search=("x86 Family 5 Model 0");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K5");
				}
				my $search=("x86 Family 5 Model 1");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K5");
				}
				my $search=("x86 Family 5 Model 2");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K5");
				}
				my $search=("x86 Family 5 Model 3");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K5");
				}
				my $search=("x86 Family 5 Model 6");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K6");
				}
				my $search=("x86 Family 5 Model 7");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K6");
				}
				my $search=("x86 Family 5 Model 8");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K6-2");
				}
				my $search=("x86 Family 5 Model 9");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K6-III");
				}
				my $search=("x86 Family 5 Model D");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD K6-2+ or K6-III+");
				}
				my $search=("x86 Family 6 Model 1");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon");
				}
				my $search=("x86 Family 6 Model 2");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon");
				}
				my $search=("x86 Family 6 Model 3");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Duron");
				}
				my $search=("x86 Family 6 Model 4");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon");
				}
				my $search=("x86 Family 6 Model 6");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon");
				}
				my $search=("x86 Family 6 Model 7");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Duron");
				}
				my $search=("x86 Family 6 Model 8");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon XP");
				}
				my $search=("x86 Family 6 Model A");
				my $pos=index("$arch","$search");
				if ($pos > -1) {
					$value=("AMD Athlon XP");
				}
			}
			$value = ("Cyrix Processor") if ($vendor eq "CyrixInstead");
			$value = ("Transmeta") if ($vendor eq "GenuineTMx86");
		} else {
			$hkey->QueryValueEx("Identifier", $type, $value);
		}

	}
#	Xchat::print "CentralProcessor: $value\n";
	return $value;
}

sub gx::get_uptime {
	my $uptime=Win32::GetTickCount();
	return $uptime;
}

sub gx::get_up {
	my $uptime=Win32::GetTickCount();
	$uptime=($uptime/1000);
	return $uptime;
}

sub gx::get_os {
	my $version=get_win_version();
	return $version;
}

##################################################
# Makes an WinAmp Query through the httpQ plugin #
# of form http://urhost:port/command?pass=urpass #
# or http://urhost:port/command?pass=urpass&a=arg#
##################################################
sub query_httpq {
	my $cmd=shift;
	my $arg=shift;
	$cmd=lc($cmd);
	my $tmp;
	if ($arg) {
		$tmp=get("http://$httpqhost:$httpqport/$cmd?p=$httpqpass&$arg");
	} else {
		$tmp=get("http://$httpqhost:$httpqport/$cmd?p=$httpqpass");
	}
	return $tmp;
}

##################################################
# Displays winamp Status if no arguments are     #
# given. If argument NEXT,PREV,PLAY,STOP or      #
# PAUSE is given it operates winamp accordingly. #
# REQUIREMENTS are the httpQ plugins for WinAmp  #
# (http://www.csc.uvic.ca/~karvanit/) and the    #
# LWP package [libwww-perl available from        #
# http://www.linpro.no/lwp/] and it's included   #
# in standard ActiveStatePerl destributions      #
# (http://www.activestate.com/).                 #
##################################################
sub action_winamp_status {
	my $cmd=shift;
	my $param=shift;
	my $chan="";
	my $context=shift;

	my $songnum;
	my $listlength;

	my $output;

	if ($cmd eq "/ME" or $cmd eq "/NOTICE" or $cmd eq "/MSG") {
		if (query_httpq("ISPLAYING") == 1 or query_httpq("ISPLAYING") == 3) {
			srand;
			$songnum=query_httpq("GETPLAYLISTPOS");
			$mp3file=query_httpq("GETPLAYLISTFILE","index=$songnum");
			$mp3file=encode("iso-8859-15", $mp3file);
#			Xchat::print("MP3 file: $mp3file");
			$listlength=query_httpq("GETLISTLENGTH");
			my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
			$song=encode("iso-8859-15", $song);
			my $currentpos=query_httpq("GETOUTPUTTIME","frmt=0");
			$currentpos=($currentpos / 1000);
			my $totallength=query_httpq("GETOUTPUTTIME","frmt=1");
	                my $samplerate=query_httpq("GETINFO","info=0");
			if ($samplerate eq "44") {
				$samplerate="44.1";
                        }
			if ($samplerate eq "22") {
				$samplerate="22.05";
			}
			my $bitrate=query_httpq("GETINFO","info=1");
			my $channels=query_httpq("GETINFO","info=2");
			my ($currmin,$currsec)=GemX::ConvertSecToMin($currentpos);
			my ($totalmin,$totalsec)=GemX::ConvertSecToMin($totallength);
			my $chan = Xchat::get_info("channel");
			my $lang='';
			$samplerate=($samplerate*1000);
			Xchat::set_context($context);
			my @actions=GemX::GetPlayerActions();
			my $indx=rand @actions;
			my $text=@actions[$indx];
        	        if ($currentpos > $totallength) {
#				Xchat::set_context($context);
#				Xchat::command(".UME $text: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kiBit/s \002$samplerate\002Hz]");
				$output=("$text: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kiBit/s \002$samplerate\002Hz]");
			} else {
#				Xchat::set_context($context);
#				Xchat::command(".UME $text ".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kiBit/s \002$samplerate\002Hz]");
				$output=("$text ".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kiBit/s \002$samplerate\002Hz]");
			}
			Xchat::command(".UME $output");
		}
	} elsif ($cmd eq "WHATISPLAYING" or $cmd eq "STATUS") {
		if (query_httpq("ISPLAYING") == 0) {
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::stop","Winamp stopped...") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::stop","Winamp gestoppt...") if (GemX::GetGlobalSet("LANGUAGE") eq "DE")
		} else {
			$songnum=query_httpq("GETPLAYLISTPOS");
			$listlength=query_httpq("GETLISTLENGTH");
			my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
			my $currentpos=query_httpq("GETOUTPUTTIME","frmt=0");
			$currentpos=($currentpos / 1000);
			my $totallength=query_httpq("GETOUTPUTTIME","frmt=1");
			my $samplerate=query_httpq("GETINFO","info=0");
			if ($samplerate eq "44") {
				$samplerate="44.1";
			}
			if ($samplerate eq "22") {
				$samplerate="22.05";
			}
			my $bitrate=query_httpq("GETINFO","info=1");
			my $channels=query_httpq("GETINFO","info=2");
			my ($currmin,$currsec)=GemX::ConvertSecToMin($currentpos);
			my ($totalmin,$totalsec)=GemX::ConvertSecToMin($totallength);

			if (query_httpq("ISPLAYING") == 3) {
	        	        if ($currentpos > $totallength) {
					Xchat::set_context($context);
					Xchat::emit_print("Generic Message","$gx::pause","Paused on #".($songnum+1)."/$listlength: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
					Xchat::emit_print("Generic Message","$gx::pause","Pausiere bei #".($songnum+1)."/$listlength: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
				} else {
					Xchat::set_context($context);
					Xchat::emit_print("Generic Message","$gx::pause","Paused on #".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
					Xchat::emit_print("Generic Message","$gx::pause","Pausiere bei #".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
				}
			} elsif (query_httpq("ISPLAYING") == 1) {
	        	        if ($currentpos > $totallength) {
					Xchat::set_context($context);
					Xchat::emit_print("Generic Message","$gx::play","Playing #".($songnum+1)."/$listlength: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
					Xchat::emit_print("Generic Message","$gx::play","Spiele #".($songnum+1)."/$listlength: $song ($currmin:$currsec/\002Stream\002) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
				} else {
					Xchat::set_context($context);
					Xchat::emit_print("Generic Message","$gx::play","Playing #".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
					Xchat::emit_print("Generic Message","$gx::play","Spiele #".($songnum+1)."/$listlength: $song ($currmin:$currsec/$totalmin:$totalsec) [\002$bitrate\002kbit/s \002$samplerate\002kHz]") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
				}

			}
		}
	}
	elsif ($cmd eq "NEXT" || $cmd eq "PREV") {
		my $skip=$gx::next;
		$skip=$gx::prev if ($cmd eq "PREV");
		if (query_httpq("ISPLAYING") == 0) {
			query_httpq($cmd);
			$songnum=query_httpq("GETPLAYLISTPOS");
			$listlength=query_httpq("GETLISTLENGTH");
			my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$skip","Skipped to #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$skip","Sprang zu #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		}
		elsif (query_httpq("ISPLAYING") == 3) {
			query_httpq($cmd);
			$songnum=query_httpq("GETPLAYLISTPOS");
			$listlength=query_httpq("GETLISTLENGTH");
			my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$skip","Now playing #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$skip","Spiele jetzt #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		} elsif (query_httpq("ISPLAYING") == 1) {
			query_httpq($cmd);
			$songnum=query_httpq("GETPLAYLISTPOS");
			$listlength=query_httpq("GETLISTLENGTH");
			my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$skip","Now playing #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$skip","Spiele jetzt #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		}
	} elsif ($cmd eq "PLAY") {
		query_httpq($cmd);
		$songnum=query_httpq("GETPLAYLISTPOS");
		$listlength=query_httpq("GETLISTLENGTH");
		my $song=query_httpq("GETPLAYLISTTITLE","index=$songnum");
		Xchat::set_context($context);
		Xchat::emit_print("Generic Message","$gx::play","Now playing #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
		Xchat::emit_print("Generic Message","$gx::play","Spiele jetzt #".($songnum+1)."/$listlength: $song") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
	} elsif ($cmd eq "PAUSE") {
		if (query_httpq("ISPLAYING") == 0) {
			query_httpq($cmd);
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::stop","Winamp is not playing...") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::stop","Winamp spielt nichts ab...") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		} elsif (query_httpq("ISPLAYING") == 3) {
			query_httpq($cmd);
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::play","Winamp resumed playing....") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::play","Winamp-Wiedergabe fortgesetzt....") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		} elsif (query_httpq("ISPLAYING") == 1) {
			query_httpq($cmd);
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::pause","Winamp paused.....") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::pause","Winamp pausiert.....") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		}
	} elsif ($cmd eq "FADEOUTANDSTOP" || $cmd eq "STOP") {
		if (query_httpq("ISPLAYING") == 1 or query_httpq("ISPLAYING") == 3) {
			query_httpq($cmd);
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::stop","Winamp stopped.....") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::stop","Winamp gestoppt.....") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		} else {
			Xchat::set_context($context);
			Xchat::emit_print("Generic Message","$gx::stop","Winamp already stopped.....") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
			Xchat::emit_print("Generic Message","$gx::stop","Winamp bereits gestoppt.....") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
		}
	} else {
		Xchat::set_context($context);
		Xchat::emit_print("Generic Message","$gx::err","httpQ: Invalid or Unsupported command!") if (GemX::GetGlobalSet("LANGUAGE") eq "EN");
		Xchat::emit_print("Generic Message","$gx::err","httpQ: Ungültiges oder nicht unterstütztes Kommando!") if (GemX::GetGlobalSet("LANGUAGE") eq "DE");
	}
	$cmd;
	$param;
	$context;
	return Xchat::EAT_ALL;
}

sub amp {
	my $cmd=uc($_[0][1]);
	chomp $cmd;
	my $param=substr($_[0][2],1);
	chomp $param;
	my $chan="";
	if ($cmd eq "") {
		$cmd="/ME";
	}

	if (($cmd eq "/SEND") && $param && $mp3file) {
		if ($mp3file=~ m/^http:\/\//) {
			Xchat::command("NOTICE $param I'm listening to a stream from $mp3file");
		} else {
			Xchat::command("SEND $param \"$mp3file\"");
		}
	} elsif (($cmd eq "/SEND") && $mp3file) {
		Xchat::emit_print("Generic Message","$gx::warn","You need to specify a target user");
	} elsif (($cmd eq "/SEND") && $param) {
		Xchat::emit_print("Generic Message","$gx::warn","You need to have announced an MP3 before");
	} elsif ($cmd eq "/SEND") {
		Xchat::emit_print("Generic Message","$gx::warn","You need to specify a target user and have announced an MP3 before");
	} else {
		my $context=Xchat::get_context();

# 		Xchat::print("Phase 1");

		my $amp = threads->create(\&action_winamp_status, $cmd, $param, $context);
#		Xchat::print("Context: $context");
		my $foo=$amp->detach();
#		Xchat::print("Phase 2");
	}

	return Xchat::EAT_ALL;
}

### Begin SOUND workaround ###


##############################
# Generic sound playback     #
# &gx::playsound(filename);      #
# Checks if filename exists  #
# in sound_dir and plays it  #
# if possible                #
##############################
sub gx::playsound {
	my $filename=("@_[0]");
	my $snddir=GemX::GetSoundDir();
	my $sndfile=("$snddir\\$filename");
	my @parts=split(/\./,"$sndfile");
	my $ext=lc(@parts[$#parts]);
	my $type;
	if (("$ext" eq "wav")) {
#		$type = "waveaudio";
		$type = "MPEGVideo";
	} elsif (("$ext" eq "mid") || ("$ext" eq "midi") || ("$ext" eq "rmi")) {
		$type = "sequencer";
	} elsif (("$ext" eq "au") || ("$ext" eq "snd") || ("$ext" eq "mpa") || ("$ext" eq "aiff") || ("$ext" eq "aif") || ("$ext" eq "mp3") || ("$ext" eq "ogg") || ("$ext" eq "mp2")) {
		$type = "MPEGVideo";
	} else {
		return Xchat::EAT_NONE;
	} 

	my $lpszCommand =("status GemX-Sound mode");
        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);
	if ($lpszReturnString eq "playing") {
		if (GemX::GetGlobalSet("CTCP_SOUND_PLAYIFPLAYING")) {
		} else {
			return Xchat::EAT_ALL;
		}
	}
	$sndfile = encode("iso-8859-15", $sndfile);
 	if (-e "$sndfile") {
#		Xchat::print("Sound exists, type: ".$type);
		# Close the MCI device GemX-Sound
		my $lpszCommand =("close GemX-Sound");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

		# Open the MCI device GemX-Sound and load our soundfile
		my $lpszCommand =("open \"$sndfile\" type $type alias GemX-Sound");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

		# Set the time format for MCI device GemX-Sound to ms
		my $lpszCommand =("set GemX-Sound time format milliseconds");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

		# Get the track length from MCI device GemX-Sound
		my $lpszCommand =("status GemX-Sound length");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

		# Allow additional 5 seconds for the track to complete
		my $length=($lpszReturnString + 5000);

		# Play the sound
		my $lpszCommand =("play GemX-Sound from 0");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

		# Add a timeout handler that will close the MCI device after
		# playback completed
		Xchat::hook_timer($length,"chksnd") if !$gx::playing;
		$gx::playing = 1;
		return Xchat::EAT_ALL;
	} else {
		return Xchat::EAT_NONE;
	}
}

sub get_dccdir {
	my $dccdir=Xchat::get_prefs("dcc_completed_dir");
	$dccdir=Xchat::get_prefs("dcc_dir") if ($dccdir eq ""); 
	my $cfgdir=Xchat::get_info("xchatdir");
	$dccdir=~ s/^\./$cfgdir/;
	$dccdir=~ s/\//\\/g;
	return $dccdir;
}

sub get_dcc_temp_dir {
	my $dccdir=Xchat::get_prefs("dcc_dir");
	my $cfgdir=Xchat::get_info("xchatdir");
	$dccdir=~ s/^\./$cfgdir/;
	$dccdir=~ s/\//\\/g;
	return $dccdir;
}


########################################
# /SOUND filename                      #
# Plays a sound and sends a CTCP SOUND #
# request to the current context       #
########################################
sub sound {
	my @args=split(/ +/,"$_[1][1]");
	my $filename=$args[0];

	my $cmdline=$filename;
	chomp $cmdline;
	my @tmp=split /\s/, $cmdline;
	my $cmd=uc($tmp[0]);

	if ($cmd eq "STOP") {
		my $lpszCommand =("close GemX-Sound");
		my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);
		return Xchat::EAT_ALL;
	}

	if ($filename eq "") {
		my $snddir=GemX::GetSoundDir();
#		Xchat::print("Soundir: $snddir");
		my ( @file, $file );
		my ( @parms );
		push @parms,
			-filter =>
				[ 'Audio Files', '*.wav;*.au;*.mid;*.mp3;*.ogg;*.snd;*.midi;*.rmi;*.aiff;*.aif;*.mpa;*.mp2'
				],
			-directory => $snddir,
			-nochangedir => 1,
			-initialdir => $snddir,
			-title => 'Select a sound to play';
#		chdir($snddir);
		@file = & Win32::GUI::GetOpenFileName ( @parms );
#		chdir($gx::workdir);
		$filename=@file[0];
		my @tempfile=split(/\\/, $filename);
		$filename=@tempfile[$#tempfile];
#		Xchat::print "$filename";
#		return Xchat::EAT_NONE;
	}

	my $curchan=Xchat::get_info("channel");

#	Xchat::print("Filename: $filename");

	$filename = encode("iso-8859-15", $filename);
#	Xchat::print("Filename ISO: $filename");

	Xchat::command("CTCP $curchan SOUND $filename") if gx::playsound("$filename");
	return Xchat::EAT_ALL;
}

sub chksnd {
	# Get the status of the GemX-Sound MCI device
	my $lpszCommand =("status GemX-Sound mode");
        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);

	# Only close the device if it is stopped
	if ($lpszReturnString eq "stopped") {
		my $lpszCommand =("close GemX-Sound");
	        my ($APICallReturnValue, $lpszReturnString) = mciSendString($lpszCommand);
		$gx::playing = 0;
		return Xchat::REMOVE;
	}		
	# Else keep the timer
	return Xchat::KEEP;
}

################################
# Handles comlete DCC RECVs    #
# Previews images if image was #
# recieved, plays sounds if    #
# sound was recieved           #
################################
sub dcc_recv_complete {
#	!orgasm!.wav e:\files\in\irc/!orgasm!.wav blah325 128801
	my $inspec = $_[0][1];
	my @parts=split(/\./,"$inspec");
	my $ext=lc(@parts[$#parts]);
	$inspec=~ s/\//\\/g;
	@parts=split(/\\/,"$inspec");
	my $filename=lc(@parts[$#parts]);

	my $requested=0;
	my $pos=0;
	foreach my $files (@gx::play_list) {
		if (lc("$files") eq lc("$filename")) {
			$requested=1;
			splice @gx::play_list,$pos,1;
		}
		$pos++;
	}

	if (!$requested) {
		return Xchat::EAT_NONE;
	}

	if (("$ext" eq "mid") || ("$ext" eq "rmi") || ("$ext" eq "wav") || ("$ext" eq "au") || ("$ext" eq "snd") || ("$ext" eq "aif") || ("$ext" eq "aiff") || ("$ext" eq "mpa") || ("$ext" eq "mp3") || ("$ext" eq "mp2") || ("$ext" eq "ogg")) {
		my $snddir=GemX::GetSoundDir();
		my $indir=get_dccdir();
		my $sndfile=("$snddir\\$filename");
		my $inspec=("$indir\\$filename");
		if (-e "$sndfile") {
		} else {
			move($inspec,$sndfile);
			if (GemX::GetChanSet("CTCP_SOUND_ALLOW")) {
				gx::playsound("$filename");
			}
		}
	}
	return Xchat::EAT_NONE;
}




### End SOUND workaround ###





sub open_log {
#	my $me = Xchat::get_info("nick"); # MyNick
#	my $i2 = Xchat::get_info("channel"); # context
#	my $i3 = Xchat::get_info("server"); # Server Old: 3
#	my $i4 = Xchat::get_info("xchatdir"); # Config_Dir
#	my $i6 = Xchat::get_info("network"); # NetWork
#	my $i7 = Xchat::get_info("host"); # Server connected to Old: 7
#	my $i8 = Xchat::get_info("topic"); # Topic
	my $c = $_[0][1];
	$c =~ s/^\s+|\s+$//g;
	$c = Xchat::get_info("channel") if ("$c" eq ""); 
	$c =~ s/\\/_/g;
	$c =~ s/\//_/g;
	my $b = Xchat::get_info("xchatdir");
	$b=("$b\\xchatlogs");
	my $m = Xchat::get_prefs("irc_logmask");
	my $s = Xchat::get_info("server");
	my $n = Xchat::get_info("network");
	my $logfile = ("$b\\$m");
	$logfile=~ s/\//\\/g;
	$logfile=~ s/%s/$s/g;
	$logfile=~ s/%c/$c/g;
	$logfile=~ s/%n/$n/g;
	$logfile = encode("iso-8859-15", $logfile);
#	Xchat::print("Logfile: $logfile");
	if (-e "$logfile") {
		system(1,"start notepad \"$logfile\"");
	} else {
		if (GemX::GetGlobalSet("LANGUAGE") eq "EN") {
			Xchat::emit_print("Generic Message","$gx::warn","No logfile for $c\@$n stored.");
			Xchat::emit_print("Generic Message","$gx::info","You have probably not talked to that person before.");
		} elsif (GemX::GetGlobalSet("LANGUAGE") eq "DE") {
			Xchat::emit_print("Generic Message","$gx::warn","Kein Logbuch für $c\@$n gespeichert.");
			Xchat::emit_print("Generic Message","$gx::info","Sie haben möglicherweise noch nie mit dieser Person gesprochen.");
		}
	}	
	return Xchat::EAT_ALL;
}

sub open_data {
	my $c = $_[0][1];
	$c =~ s/^\s+|\s+$//g;
	my $b = GemX::GetGlobalSet("USERINFO");
	$b = Xchat::get_info("xchatdir") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$b=("$b\\userdata") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$c = Xchat::get_info("channel") if ("$c" eq ""); 
	my $n = Xchat::get_info("network");
	my $datadir = ("$b\\$n\\$c");
	$datadir=~ s/\//\\/g;
	my $datafile = ("$datadir\\info.txt");

	if (-e "$datafile") {
		system(1,"start notepad \"$datafile\"");
	} else {
		create_dir($datadir) if (!-d $datadir);
		for my $u (0 .. 500) {
			if (-e "$datadir") {
				system("start notepad \"$datafile\"");
				return Xchat::EAT_ALL;
			}
		}

	}	
	return Xchat::EAT_ALL;
}

sub open_pic {
	my $c = $_[0][1];
	$c =~ s/^\s+|\s+$//g;
	my $b = GemX::GetGlobalSet("USERINFO");
	$b = Xchat::get_info("xchatdir") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$b=("$b\\userdata") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$c = Xchat::get_info("channel") if ("$c" eq ""); 
	my $n = Xchat::get_info("network");
	my $picdir = ("$b\\$n\\$c");
	$picdir=~ s/\//\\/g;

	my @dir_contents;
	my $file;
	my @dirlist=();

	$picdir = encode("iso-8859-15", $picdir);

	opendir(DIR,$picdir); # || die("Cannot open directory !\n");
		@dir_contents= readdir(DIR);
	closedir(DIR);

	foreach $file (@dir_contents) {
		if(!(($file eq ".") || ($file eq ".."))) {
			my @parts=split(/\./,"$file");
			my $ext=lc(@parts[$#parts]);
			if (("$ext" eq "jpg") || ("$ext" eq "gif") || ("$ext" eq "bmp") || ("$ext" eq "jpeg") || ("$ext" eq "png") || ("$ext" eq "tif")) {
				push(@dirlist,$file);
			}
		}
	}
	if (@dirlist[0] ne "") {
		my $spec=("$picdir\\@dirlist[0]");
		Win32::FileOp::ShellExecute $spec;
	} else {
		if (GemX::GetGlobalSet("LANGUAGE") eq "EN") {
			Xchat::emit_print("Generic Message","$gx::warn","No pictures for $c\@$n stored.");
			Xchat::emit_print("Generic Message","$gx::info","Store them in directory");
			Xchat::emit_print("Generic Message","$gx::info","\002$picdir\002");
			Xchat::emit_print("Generic Message","$gx::info","and retry.");
		} elsif (GemX::GetGlobalSet("LANGUAGE") eq "DE") {
			Xchat::emit_print("Generic Message","$gx::warn","Keine Bilder für $c\@$n gespeichert.");
			Xchat::emit_print("Generic Message","$gx::info","Speichern Sie diese in dem Verzeichnis");
			Xchat::emit_print("Generic Message","$gx::info","\002$picdir\002");
			Xchat::emit_print("Generic Message","$gx::info","und versuchen Sie es erneut.");
		}
	}
	return Xchat::EAT_ALL;
}

sub get_appdata {
	my $hkey;
	$::HKEY_CURRENT_USER->Open("Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",$hkey) or die "Can't open registry";

	my ($type, $value);
	$hkey->QueryValueEx("AppData", $type, $value) or die "No APPDATA: $^E";
	return $value;
}

sub get_flashappdata {
	my $hkey;
	$::HKEY_LOCAL_MACHINE->Open("SOFTWARE\\FlashFXP",$hkey) or return 0;

	my ($type, $value);
	$hkey->QueryValueEx("APPDATA", $type, $value) or return 0;
	return $value;
}

sub get_flashdir {
	my $hkey;
	$::HKEY_CLASSES_ROOT->Open("FlashFXP_PIcon\\DefaultIcon",$hkey);

	my ($type, $value);
	$hkey->QueryValueEx("", $type, $value) or die "No Path: $^E";
	my @value=split(/\,/,$value);
	$value=substr(@value[0],0,(length($value)-15));
	Xchat::print "Path: $value\n";
	return $value;
}



sub get_ffxp_dir {
	my $appdata="";
	my $flash_appdata=get_flashappdata;
	
	if ($flash_appdata) {
		$appdata=get_appdata();
		$appdata="$appdata\\FlashFXP\\";
	} else {
		$appdata=get_flashdir();
	}
	return $appdata;
}

sub fxpfxped {
	my $fstats=get_ffxp_dir();
	my $ffxp = new Config::IniFiles( -file => "$fstats\\stats.dat" );
	my $flashfxpbytes = $ffxp->val( '#Total#', 'X' ) if $ffxp->val( '#Total#', 'X' );
	my $flashfxpdate2 = $ffxp->val( '#Total#', 'LR' ) if $ffxp->val( '#Total#', 'LR' );
	# FlashFXP stores its Last Reset (LR) time in days since 1.1.1900 0:00
	# For further calculation we need a unix time stamp which is stored
	# in seconds since 1.1.1970 0:00
	# So we subtract 25569 (days between 1.1.1900 and 1.1.1970) and
	# multiply with 86400 (seconds per day) to get a Unix time stamp:
	$flashfxpdate2 = (($flashfxpdate2 - 25569) * 86400);
	my $offset = -2;
	$flashfxpdate2 = ($flashfxpdate2 + ($offset * 3600));
	my $flashfxpdate1 = strftime "%a dem \002%d\. %b %Y\002", localtime($flashfxpdate2);
	my $timediff = (time - $flashfxpdate2);
	my $flashfxpratekbps = sprintf("%.2f", (($flashfxpbytes / 1024) / $timediff));
	my $flashfxpratembpm = sprintf("%.2f", (($flashfxpbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashfxpgb = sprintf("%.0f", ($flashfxpbytes/1024/1024/1024));
	my $flashfxpmb = sprintf("%.0f", ($flashfxpbytes/1024/1024));
	my $flashfxpkb = sprintf("%.0f", ($flashfxpbytes/1024));
	if ($flashfxpbytes > 1000000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashfxpgb gB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 1000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashfxpmb mB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 1000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashfxpkb kB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 0) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashfxpbytes kB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich nichts mit FlashFXP geflashed (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}
	return Xchat::EAT_ALL;
}

sub fxpgeuppt {
	my $fstats=get_ffxp_dir();
	my $ffxp = new Config::IniFiles( -file => "$fstats\\stats.dat" );
	my $flashupbytes = $ffxp->val( '#Total#', 'FU' ) if $ffxp->val( '#Total#', 'FU' );
	my $flashfxpdate2 = $ffxp->val( '#Total#', 'LR' ) if $ffxp->val( '#Total#', 'LR' );
	# FlashFXP stores its Last Reset (LR) time in days since 1.1.1900 0:00
	# For further calculation we need a unix time stamp which is stored
	# in seconds since 1.1.1970 0:00
	# So we subtract 25569 (days between 1.1.1900 and 1.1.1970) and
	# multiply with 86400 (seconds per day) to get a Unix time stamp:
	$flashfxpdate2 = (($flashfxpdate2 - 25569) * 86400);
	my $offset = -2;
	$flashfxpdate2 = ($flashfxpdate2 + ($offset * 3600));
	my $flashfxpdate1 = strftime "%a dem \002%d\. %b %Y\002", localtime($flashfxpdate2);
	my $timediff = (time - $flashfxpdate2);
	my $flashupratekbps = sprintf("%.2f", (($flashupbytes / 1024) / $timediff));
	my $flashupratembpm = sprintf("%.2f", (($flashupbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashupgb = sprintf("%.0f", ($flashupbytes/1024/1024/1024));
	my $flashupmb = sprintf("%.0f", ($flashupbytes/1024/1024));
	my $flashupkb = sprintf("%.0f", ($flashupbytes/1024));
	if ($flashupbytes > 1000000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashupgb gB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} elsif ($flashupbytes > 1000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashupmb mB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag.\002");
	} elsif ($flashupbytes > 1000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashupkb kB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} elsif ($flashupbytes > 0) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashupbytes kB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich nichts mit FlashFXP geuppt (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}
	return Xchat::EAT_ALL;
}

sub fxpgesaugt {
	my $fstats=get_ffxp_dir();
	my $ffxp = new Config::IniFiles( -file => "$fstats\\stats.dat" );
	my $flashdnbytes = $ffxp->val( '#Total#', 'FD' ) if $ffxp->val( '#Total#', 'FD' );
	my $flashfxpdate2 = $ffxp->val( '#Total#', 'LR' ) if $ffxp->val( '#Total#', 'LR' );
	# FlashFXP stores its Last Reset (LR) time in days since 1.1.1900 0:00
	# For further calculation we need a unix time stamp which is stored
	# in seconds since 1.1.1970 0:00
	# So we subtract 25569 (days between 1.1.1900 and 1.1.1970) and
	# multiply with 86400 (seconds per day) to get a Unix time stamp:
	$flashfxpdate2 = (($flashfxpdate2 - 25569) * 86400);
	my $offset = -2;
	$flashfxpdate2 = ($flashfxpdate2 + ($offset * 3600));
	my $flashfxpdate1 = strftime "%a dem \002%d\. %b %Y\002", localtime($flashfxpdate2);
	my $timediff = (time - $flashfxpdate2);
	my $flashdnratekbps = sprintf("%.2f", (($flashdnbytes / 1024) / $timediff));
	my $flashdnratembpm = sprintf("%.2f", (($flashdnbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashdngb = sprintf("%.0f", ($flashdnbytes/1024/1024/1024));
	my $flashdnmb = sprintf("%.0f", ($flashdnbytes/1024/1024));
	my $flashdnkb = sprintf("%.0f", ($flashdnbytes/1024));
	if ($flashdnbytes > 1000000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashdngb gB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 1000000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashdnmb mB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 1000) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashdnkb kB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 0) {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich \002$flashdnbytes Bytes\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Seit $flashfxpdate1 habe ich nichts mit FlashFXP gesaugt (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}
	return Xchat::EAT_ALL;
}

sub fxpalle {
	my $fstats=get_ffxp_dir();
	my $ffxp = new Config::IniFiles( -file => "$fstats\\stats.dat" );
	my $flashfxpbytes = $ffxp->val( '#Total#', 'X' ) if $ffxp->val( '#Total#', 'X' );
	my $flashupbytes = $ffxp->val( '#Total#', 'FU' ) if $ffxp->val( '#Total#', 'FU' );
	my $flashdnbytes = $ffxp->val( '#Total#', 'FD' ) if $ffxp->val( '#Total#', 'FD' );
	my $flashfxpdate2 = $ffxp->val( '#Total#', 'LR' ) if $ffxp->val( '#Total#', 'LR' );
	# FlashFXP stores its Last Reset (LR) time in days since 1.1.1900 0:00
	# For further calculation we need a unix time stamp which is stored
	# in seconds since 1.1.1970 0:00
	# So we subtract 25569 (days between 1.1.1900 and 1.1.1970) and
	# multiply with 86400 (seconds per day) to get a Unix time stamp:
	$flashfxpdate2 = (($flashfxpdate2 - 25569) * 86400);
	my $offset = -2;
	$flashfxpdate2 = ($flashfxpdate2 + ($offset * 3600));
	my $flashfxpdate1 = strftime "%a dem %d\. %b %Y", localtime($flashfxpdate2);
	my $timediff = (time - $flashfxpdate2);

	Xchat::command(".USAY \002Meine FlashFXP Statistik seit $flashfxpdate1\!\002");

	my $flashfxpratekbps = sprintf("%.2f", (($flashfxpbytes / 1024) / $timediff));
	my $flashfxpratembpm = sprintf("%.2f", (($flashfxpbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashfxpgb = sprintf("%.0f", ($flashfxpbytes/1024/1024/1024));
	my $flashfxpmb = sprintf("%.0f", ($flashfxpbytes/1024/1024));
	my $flashfxpkb = sprintf("%.0f", ($flashfxpbytes/1024));
	if ($flashfxpbytes > 1000000000) {
		Xchat::command(".USAY Ich habe \002$flashfxpgb gB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 1000000) {
		Xchat::command(".USAY Ich habe \002$flashfxpmb mB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 1000) {
		Xchat::command(".USAY Ich habe \002$flashfxpkb kB\002 mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} elsif ($flashfxpbytes > 0) {
		Xchat::command(".USAY Ich habe \002$flashfxpbytes\002 Bytes mit FlashFXP geflashed. Das ergibt einen Durchschnitt von \002$flashfxpratekbps kB/s\002 oder \002$flashfxpratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Ich habe nichts mit FlashFXP geflashed (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}



	my $flashupratekbps = sprintf("%.2f", (($flashupbytes / 1024) / $timediff));
	my $flashupratembpm = sprintf("%.2f", (($flashupbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashupgb = sprintf("%.0f", ($flashupbytes/1024/1024/1024));
	my $flashupmb = sprintf("%.0f", ($flashupbytes/1024/1024));
	my $flashupkb = sprintf("%.0f", ($flashupbytes/1024));
	if ($flashupbytes > 1000000000) {
		Xchat::command(".USAY Ich habe \002$flashupgb gB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} elsif ($flashupbytes > 1000000) {
		Xchat::command(".USAY Ich habe \002$flashupmb mB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} elsif ($flashupbytes > 1000) {
		Xchat::command(".USAY Ich habe \002$flashupkb kB\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} elsif ($flashupbytes > 0) {
		Xchat::command(".USAY Ich habe \002$flashupbytes Bytes\002 mit FlashFXP geuppt. Das ergibt einen Durchschnitt von \002$flashupratekbps kB/s\002 oder \002$flashupratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Ich habe nichts mit FlashFXP geuppt (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}

	my $flashdnratekbps = sprintf("%.2f", (($flashdnbytes / 1024) / $timediff));
	my $flashdnratembpm = sprintf("%.2f", (($flashdnbytes / 1024 / 1024) / ($timediff / 60 / 60 / 24)));
	my $flashdngb = sprintf("%.0f", ($flashdnbytes/1024/1024/1024));
	my $flashdnmb = sprintf("%.0f", ($flashdnbytes/1024/1024));
	my $flashdnkb = sprintf("%.0f", ($flashdnbytes/1024));
	if ($flashdnbytes > 1000000000) {
		Xchat::command(".USAY Ich habe \002$flashdngb gB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 1000000) {
		Xchat::command(".USAY Ich habe \002$flashdnmb mB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 1000) {
		Xchat::command(".USAY Ich habe \002$flashdnkb kB\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} elsif ($flashdnbytes > 0) {
		Xchat::command(".USAY Ich habe \002$flashdnbytes Bytes\002 mit FlashFXP gesaugt. Das ergibt einen Durchschnitt von \002$flashdnratekbps kB/s\002 oder \002$flashdnratembpm mB/Tag\002.");
	} else {
		Xchat::command(".USAY Ich habe nichts mit FlashFXP gesaugt (Oder FlashFXP hat mal wieder das Loggen vergessen).");
	}
	return Xchat::EAT_ALL;
}

sub complete_handler {
#	!orgasm!.wav e:\files\in\irc/!orgasm!.wav blah325 128801
	my $inspec = $_[0]->[1];
	$inspec=~ s/\//\\/g;

	my @specs=split(/\\/,"$inspec");
	my $filename=(@specs[$#specs]);

	my @parts=split(/\./,"$inspec");
	my $ext=lc(@parts[$#parts]);

	if (("$ext" eq "jpg") || ("$ext" eq "jpeg") || ("$ext" eq "gif") || ("$ext" eq "bmp") || ("$ext" eq "jpeg") || ("$ext" eq "png") || ("$ext" eq "tif") || ("$ext" eq "tiff")) {
		if (GemX::GetGlobalSet("PREVIEW")) {
			my $dccdir=get_dccdir();
			my $filespec=($dccdir."\\".$filename);
			Win32::FileOp::ShellExecute $filespec;
		}
	}
	return Xchat::EAT_NONE;

}

sub fetch_pics {
	my $c = $_[0][1];
	$c =~ s/^\s+|\s+$//g;
	my $b = GemX::GetGlobalSet("USERINFO");
	$b = Xchat::get_info("xchatdir") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$b=("$b\\userdata") if ((GemX::GetGlobalSet("USERINFO")) eq "");
	$c = Xchat::get_info("channel") if ("$c" eq ""); 
	my $n = Xchat::get_info("network");
	my $picdir = ("$b\\$n\\$c");
	$picdir=~ s/\//\\/g;

	my $indir=get_dccdir();
	my $inpath;
	my ( @file, $file );
	my ( @parms );
	push @parms,
		-multisel => 10,
		-filter =>
			[ 'Pictures', '*.jpg;*.jpeg;*.tif;*.tiff;*.png;*.gif;*.bmp'
			],
		-directory => $indir,
		-initialdir => $indir,
		-nochangedir => 1,
		-title => 'Select a file';
	@file;

#	chdir($indir);
	@file = & Win32::GUI::GetOpenFileName ( @parms );
#	chdir($gx::workdir);

	if (!@file) {
		Xchat::emit_print("Generic Message","$gx::warn","No file selected");
		return Xchat::EAT_ALL;
	}

	if ($#file gt 1) {
		$inpath=@file[0];
		splice @file,0,1;
	} else {
		my ($volume,$directories,$file) = File::Spec->splitpath( @file[0] );
		$inpath =  File::Spec->catpath( $volume,$directories );
	}

#	Xchat::print("Inpath: $inpath");

	my @dir_contents;
	my $file;
	my @dirlist=();

	opendir(DIR,$picdir) || create_dir($picdir);
	closedir(DIR);

	for my $file (@file) {
		my @tempfile=split(/\\/, $file);
		my $filename=@tempfile[$#tempfile];
		my $inspec=("$inpath\\$filename");
		my $outspec=("$picdir\\$filename");
#		Xchat::print("Copying $inspec to $outspec");
		move($inspec,$outspec);
	}
	return Xchat::EAT_ALL;

}

sub create_dir {
	my $dir=$_[0];
	my @segments=split(/\\/,"$dir");
	my $createdir=shift(@segments);
	foreach my $seg (@segments) {
		$createdir=("$createdir\\$seg");
		mkdir("$createdir") if (!-d $createdir);
		wait;
	}
	return 1 if (-d $dir);
	return 0;
}

sub figfont {
	my $figdir=GemX::GetGlobalSet("FIGDIR");
		my ( @file, $file );
		my ( @parms );
		push @parms,
			-filter =>
				[ 'FIGlet fonts', '*.flf'
				],
			-directory => $figdir,
			-initialdir => $figdir,
			-nochangedir => 1,
			-title => 'Select a font';
#		chdir($figdir);
		@file = & Win32::GUI::GetOpenFileName ( @parms );
#		chdir($gx::workdir);
		my $filename=@file[0];

	if ($filename) {
		my ($volume,$directories,$file) = File::Spec->splitpath( @file[0] );
		my $newdir =  File::Spec->catpath( $volume,$directories );
		GemX::SetChanSet("FIGDIR",$newdir,2);
		$file=~ s/\.flf$//i;
		GemX::SetChanSet("FIGFONT",$file,2);
	}


	return Xchat::EAT_ALL;
}

sub open_dcc {
	my $indir=get_dccdir();

	Win32::FileOp::ShellExecute $indir;

	return Xchat::EAT_ALL;
}

sub pizza {
	my $plz = $_[0][1];
	my $url = ("http://www.webimbiss.de/suche.php5?plz=".$plz."&Button=go!");

	Win32::FileOp::ShellExecute $url;

	return Xchat::EAT_ALL;
}

sub wmp {
	my $wmpdir="$ENV{PROGRAMFILES}\\Windows Media Player";
	my $wmpnpfile="np_info.txt";

	if (open(WMPNP,"<$wmpdir/$wmpnpfile")) {
		my $wmpnp = <WMPNP>;
		close(WMPNP);

		my $np;
		my $matched;

		$matched=$wmpnp=~ /^.*?\-\s*(.*)$/;
		if ($matched) {
			Xchat::command(".UME spielt gerade im \cBWindows Media Player\cB ab: \cB\cC11$1\cO");
		}
	} else {
	}

	return Xchat::EAT_ALL;
}

sub get_emule_path {
	my $cmd=$_[0][1];
	$cmd=uc($cmd);
	$cmd=~ s/^\///;
	$cmd=~ s/^\-//;
	my $full;

	my $loading;
	my $complete;
	my $paused;
	my $waiting;
	my $other;

	if (($cmd eq "FULL") || ($cmd eq "VERBOSE") || ($cmd eq "V") || ($cmd eq "LONG")) {
		$full=1;
	} elsif ($cmd) {
		$loading=$cmd=~ m/L/g;
		$complete=$cmd=~ m/C/g;
		$paused=$cmd=~ m/P/g;
		$waiting=$cmd=~ m/W/g;
		$other=$cmd=~ m/O/g;
	} else {
		$loading=1;
	}

	my $hkey;
	$::HKEY_CLASSES_ROOT->Open("ed2k\\shell\\open\\command",$hkey);

	my ($type, $value);
	
	$hkey->QueryValueEx("", $type, $value);

	$value=~ m/\"(.*?)\"/;

	my $emule=$1;

	if (($emule eq "") || ($emule eq "\%1")) {
		$value=~ m/^(.*?)\s/;
		$emule=$1;
	}

	if ($emule eq "") {
		Xchat::emit_print("Generic Message","$gx::warn","eMule not found");
		return Xchat::EAT_ALL;
	}

	$emule=~ m/^(.*)\\/;

	my $epath=$1;

#	Xchat::print("Sending status to $emule");

	system("\"$emule\" status");

	my $file = "${epath}\\status.log";	# Name the file

#	Xchat::print("Reading $file for stats");
	open(EMULE, $file);			# Open the file
	my @lines = <EMULE>;			# Read it into an array
	close(EMULE);				# Close the file

	my $status=shift(@lines);
	chomp($status);
	my $speed=shift(@lines);
	chomp($speed);

#	Xchat::print("Emule stats read; $#lines Lines");

	if (@lines) {
		my $num=$#lines+1;
		Xchat::commandf(".USAY \00300\002eMule Stats");
		Xchat::commandf(".USAY \00311Server connection\017: \00308$status \00307~ \00311Speed\017: \00308$speed \00307~ \00311Total \00308$num \00311downloads");


		@lines=sort(@lines);
		for my $n (0 .. ($#lines)) {
			@lines[$n]=~ m/^(.*)\[(.*)\]\s(.*) \- (.*)$/;
			my $name=$1;
			my $percent=$2;
			my $sources=$3;
			my $stat=decode("ISO-8859-15",$4);
			$name=~ s/\s*$//;

			chomp $stat;

			my $tstat=lc($stat);

			if (($tstat eq "ladend") || ($tstat eq "downloading")) {
				if (($full) || ($loading)) {
					$num--;
					Xchat::commandf(".USAY \00309".'%-70.70s'."\017: \00308".'%6s'." \00307~ \00308".'%10s'." \00307~\00308 ".'%s',$name,$percent,$sources,$stat);
				}
			} elsif (($tstat eq "warte") || ($tstat eq "waiting")) {
				if (($full) || ($waiting)) {
					$num--;
					Xchat::commandf(".USAY \00309".'%-70.70s'."\017: \00308".'%6s'." \00307~ \00308".'%10s'." \00307~\00308 ".'%s',$name,$percent,$sources,$stat);
				}
			} elsif (($tstat eq "vollständig") || ($tstat=~ m/Wird beendet/) || ($tstat eq "complete") || ($tstat eq "completing")) {
		# ä Ä = ae
		# ö Ö = oe
		# ü Ü = ue
		# ß = ss
				if (($full) || ($complete)) {
					$num--;
					Xchat::commandf(".USAY \00309".'%-70.70s'."\017: \00308".'%6s'." \00307~ \00308".'%10s'." \00307~\00308 ".'%s',$name,$percent,$sources,$stat);
				}
			} elsif (($tstat eq "angehalten") || ($tstat eq "gestoppt") || ($tstat eq "paused") || ($tstat eq "stopped")) {
				if (($full) || ($paused)) {
					$num--;
					Xchat::commandf(".USAY \00309".'%-70.70s'."\017: \00308".'%6s'." \00307~ \00308".'%10s'." \00307~\00308 ".'%s',$name,$percent,$sources,$stat);
				}
			} else {
				if (($full) || ($other)) {
					$num--;
					Xchat::commandf(".USAY \00309".'%-70.70s'."\017: \00308".'%6s'." \00307~ \00308".'%10s'." \00307~\00308 ".'%s',$name,$percent,$sources,$stat);
				}
			}
		}
		if ($num) {
			Xchat::command(".USAY \00308$num \00311downloads in unlisted states.");
		}

	}
	return Xchat::EAT_ALL;

}

sub ascii {
	my @args=split(/ +/,"$_[1][1]");
	my $filename=$args[0];

	my $asciidir=GemX::GetGlobalSet("ASCIIDIR");
	$asciidir||=Xchat::get_info("xchatdir")."\\ascii";


	if (-e "$filename") {
	} elsif ((-e "$asciidir\\$filename") && $filename) {
		$filename=$asciidir."\\".$filename;
	} else {
		my ( @file, $file );
		my ( @parms );
		push @parms,
			-filter =>
				[ 'ASCII Artz', '*.ascii;*.txt'
				],
			-directory => $asciidir,
			-nochangedir => 1,
			-initialdir => $asciidir,
			-title => 'Select an ASCII art to post';
		@file = & Win32::GUI::GetOpenFileName ( @parms );
		$filename=@file[0];
	}

	if (-e "$filename") {
		my ($volume,$directories,$file) = File::Spec->splitpath( $filename );
		my $newdir =  File::Spec->catpath( $volume,$directories );

		GemX::SetChanSet("ASCIIDIR",$newdir,2);

		$filename = encode("iso-8859-15", $filename);
		open(ascii, "<$filename");
		while(<ascii>)	{
			chomp $_;
			Xchat::command(".USAY ".$_);
		}
		close(ascii);
	}

	return Xchat::EAT_ALL;
}

sub vgirlhd {
	my $vgirlhd = Win32::GUI::FindWindow("CicMarshalWndClass","");
#	my $vgirlhd = Win32::GUI::FindWindow("VGHD Player","");
#	my $vgirlhd = Win32::GUI::FindWindow("Vghd","");
#	my $vgirlhd = Win32::GUI::FindWindow("tooltipsclass_32","");
#	my $vgirlhd = Win32::GUI::FindWindow("TipBox","");
	my $skip=$gx::next;
	if ($vgirlhd) {
		my $GetWindowText = new Win32::API('user32', 'GetWindowText', 'NPI', 'I');
		#my $GetWindowText = new Win32::API('user32', 'SendMessage', 'NIIP', 'I');
		my $vgirlhdcard = " " x 40;
		my $success=$GetWindowText->Call($vgirlhd, $vgirlhdcard, 40);
		#my $success=$GetWindowText->Call($vgirlhd, 13, 40, $vgirlhdcard);
		Xchat::emit_print("Generic Message","$skip","Window handle is ".$vgirlhd." Success =".$vgirlhdcard);
	} else {
		Xchat::emit_print("Generic Message","$skip","Window handle is ".$vgirlhd);
	}
 	return Xchat::EAT_ALL;
}






